@@ -72,7 +72,7 @@ defmodule DateTime do
   rules, ultimately affecting the result. For example, a country may
   choose to enter or abandon "Daylight Saving Time", which is a
   process where we adjust the clock one hour forward or one hour
-  back once per year. Whenener the rules change, the exact instant
+  back once per year. Whenever the rules change, the exact instant
   that 2:30 AM in Polish time will be in Brazil may change.
 
   In other words, whenever working with future DateTimes, there is
@@ -88,12 +88,13 @@ defmodule DateTime do
   time zone observes "Daylight Saving Time", they will move their
   clock forward once a year. When this happens, there is a whole
   hour that does not exist. Then, when they move the clock back,
-  there is a certain hour that will happen twice. So if you want
-  to schedule a meeting when this shift back happens, you would
-  need to explicitly say which of the 2:30 AM you precisely mean.
-  Applications that are date and time sensitive, need to take
-  these scenarios into account and correctly communicate them to
-  users.
+  there is a certain hour that will happen twice. So if you want to
+  schedule a meeting when this shift back happens, you would need to
+  explicitly say which occurence of 2:30 AM you mean: the one in
+  "Summer Time", which occurs before the shift, or the one
+  in "Standard Time", which occurs after it. Applications that are
+  date and time sensitive need to take these scenarios into account
+  and correctly communicate them to users.
 
   The good news is: Elixir contains all of the building blocks
   necessary to tackle those problems. The default timezone database
@@ -103,6 +104,24 @@ defmodule DateTime do
   query the database and return the relevant information. For
   example, look at how `DateTime.new/4` returns different results
   based on the scenarios described in this section.
+
+  ## Converting between timezones
+
+  Bearing in mind the cautions above, and assuming you've brought in a full
+  timezone database, here are some examples of common shifts between time
+  zones.
+
+      # Local time to UTC
+      new_york = DateTime.from_naive!(~N[2023-06-26T09:30:00], "America/New_York")
+      #=> #DateTime<2023-06-26 09:30:00-04:00 EDT America/New_York>
+
+      utc = DateTime.shift_zone!(new_york, "Etc/UTC")
+      #=> ~U[2023-06-26 13:30:00Z]
+
+      # UTC to local time
+      DateTime.shift_zone!(utc, "Europe/Paris")
+      #=> #DateTime<2023-06-26 15:30:00+02:00 CEST Europe/Paris>
+
   """
 
   @enforce_keys [:year, :month, :day, :hour, :minute, :second] ++
@@ -174,7 +193,7 @@ defmodule DateTime do
   end
 
   @doc """
-  Returns the current datetime in UTC, supporting 
+  Returns the current datetime in UTC, supporting
   a specific calendar and precision.
 
   If you want the current time in Unix seconds,
@@ -1189,7 +1208,7 @@ defmodule DateTime do
   end
 
   @doc """
-  Converts to ISO8601 specifying both a calendar and a mode.
+  Converts from ISO8601 specifying both a calendar and a mode.
 
   See `from_iso8601/2` for more information.
 
@@ -1427,7 +1446,7 @@ defmodule DateTime do
   end
 
   @doc """
-  Returns true if the first datetime is strictly earlier than the second.
+  Returns `true` if the first datetime is strictly earlier than the second.
 
   ## Examples
 
@@ -1446,7 +1465,7 @@ defmodule DateTime do
   end
 
   @doc """
-  Returns true if the first datetime is strictly later than the second.
+  Returns `true` if the first datetime is strictly later than the second.
 
   ## Examples
 
@@ -1516,6 +1535,12 @@ defmodule DateTime do
         %{utc_offset: utc_offset2, std_offset: std_offset2} = datetime2,
         unit
       ) do
+    if not is_integer(unit) and
+         unit not in ~w(second millisecond microsecond nanosecond)a do
+      raise ArgumentError,
+            "unsupported time unit. Expected :day, :hour, :minute, :second, :millisecond, :microsecond, :nanosecond, or a positive integer, got #{inspect(unit)}"
+    end
+
     naive_diff =
       (datetime1 |> to_iso_days() |> Calendar.ISO.iso_days_to_unit(unit)) -
         (datetime2 |> to_iso_days() |> Calendar.ISO.iso_days_to_unit(unit))
@@ -1532,10 +1557,10 @@ defmodule DateTime do
   `t:System.time_unit/0`. It defaults to `:second`. Negative values
   will move backwards in time.
 
-  This function always consider the unit to be computed according
+  This function always considers the unit to be computed according
   to the `Calendar.ISO`.
 
-  This function uses relies on a contiguous representation of time,
+  This function relies on a contiguous representation of time,
   ignoring the wall time and timezone changes. For example, if you add
   one day when there are summer time/daylight saving time changes,
   it will also change the time forward or backward by one hour,
