@@ -471,6 +471,12 @@ defmodule NaiveDateTime do
         unit
       )
       when is_integer(amount_to_add) do
+    if not is_integer(unit) and
+         unit not in ~w(second millisecond microsecond nanosecond)a do
+      raise ArgumentError,
+            "unsupported time unit. Expected :day, :hour, :minute, :second, :millisecond, :microsecond, :nanosecond, or a positive integer, got #{inspect(unit)}"
+    end
+
     ppd = System.convert_time_unit(86400, :second, unit)
     precision = max(Calendar.ISO.time_unit_to_precision(unit), precision)
 
@@ -554,6 +560,12 @@ defmodule NaiveDateTime do
               "and thus the result would be ambiguous"
     end
 
+    if not is_integer(unit) and
+         unit not in ~w(second millisecond microsecond nanosecond)a do
+      raise ArgumentError,
+            "unsupported time unit. Expected :day, :hour, :minute, :second, :millisecond, :microsecond, :nanosecond, or a positive integer, got #{inspect(unit)}"
+    end
+
     units1 = naive_datetime1 |> to_iso_days() |> Calendar.ISO.iso_days_to_unit(unit)
     units2 = naive_datetime2 |> to_iso_days() |> Calendar.ISO.iso_days_to_unit(unit)
     units1 - units2
@@ -1073,7 +1085,7 @@ defmodule NaiveDateTime do
   end
 
   @doc """
-  Returns true if the first `NaiveDateTime` is strictly earlier than the second.
+  Returns `true` if the first `NaiveDateTime` is strictly earlier than the second.
 
   ## Examples
 
@@ -1092,7 +1104,7 @@ defmodule NaiveDateTime do
   end
 
   @doc """
-  Returns true if the first `NaiveDateTime` is strictly later than the second.
+  Returns `true` if the first `NaiveDateTime` is strictly later than the second.
 
   ## Examples
 
@@ -1212,7 +1224,7 @@ defmodule NaiveDateTime do
 
       datetime
       |> NaiveDateTime.beginning_of_day()
-      |> DateTime.from_naive(datetime.timezone)
+      |> DateTime.from_naive(datetime.time_zone)
 
   Note that the beginning of the day may not exist or be ambiguous
   in a given timezone, so you must handle those cases accordingly.
@@ -1239,7 +1251,7 @@ defmodule NaiveDateTime do
 
       datetime
       |> NaiveDateTime.end_of_day()
-      |> DateTime.from_naive(datetime.timezone)
+      |> DateTime.from_naive(datetime.time_zone)
 
   Note that the end of the day may not exist or be ambiguous
   in a given timezone, so you must handle those cases accordingly.
