@@ -31,7 +31,21 @@ defmodule Exception do
   @type arity_or_args :: non_neg_integer | list
   @type location :: keyword
 
+  @doc """
+  Receives the arguments given to `raise/2` and returns the exception struct.
+
+  The default implementation accepts either a set of keyword arguments
+  that is merged into the struct or a string to be used as the exception's message.
+  """
   @callback exception(term) :: t
+
+  @doc """
+  Receives the exception struct and must return its message.
+
+  Most commonly exceptions have a message field which by default is accessed
+  by this function. However, if an exception does not have a message field,
+  this function must be explicitly implemented.
+  """
   @callback message(t) :: String.t()
 
   @doc """
@@ -58,6 +72,7 @@ defmodule Exception do
   @doc """
   Gets the message for an `exception`.
   """
+  @spec message(t) :: String.t()
   def message(%module{__exception__: true} = exception) do
     try do
       module.message(exception)
@@ -647,6 +662,7 @@ defmodule Exception do
   A stacktrace must be given as an argument. If not, the stacktrace
   is retrieved from `Process.info/2`.
   """
+  @spec format_stacktrace(stacktrace | nil) :: String.t()
   def format_stacktrace(trace \\ nil) do
     trace =
       if trace do
@@ -673,6 +689,7 @@ defmodule Exception do
       #=> "#Function<...>/1"
 
   """
+  @spec format_fa(fun, arity) :: String.t()
   def format_fa(fun, arity) when is_function(fun) do
     "#{inspect(fun)}#{format_arity(arity)}"
   end
@@ -697,6 +714,7 @@ defmodule Exception do
   where func is the name of the enclosing function. Convert to
   "anonymous fn in func/arity"
   """
+  @spec format_mfa(module, atom, arity_or_args) :: String.t()
   def format_mfa(module, fun, arity) when is_atom(module) and is_atom(fun) do
     case Code.Identifier.extract_anonymous_fun_parent(fun) do
       {outer_name, outer_arity} ->
@@ -736,6 +754,7 @@ defmodule Exception do
       ""
 
   """
+  @spec format_file_line(String.t() | nil, non_neg_integer | nil, String.t()) :: String.t()
   def format_file_line(file, line, suffix \\ "") do
     cond do
       is_nil(file) -> ""
@@ -767,6 +786,12 @@ defmodule Exception do
       ""
 
   """
+  @spec format_file_line_column(
+          String.t() | nil,
+          non_neg_integer | nil,
+          non_neg_integer | nil,
+          String.t()
+        ) :: String.t()
   def format_file_line_column(file, line, column, suffix \\ "") do
     cond do
       is_nil(file) -> ""
@@ -776,19 +801,199 @@ defmodule Exception do
     end
   end
 
-  @doc false
-  def format_snippet(snippet, error_line) do
-    line_digits = error_line |> Integer.to_string() |> byte_size()
-    placeholder = String.duplicate(" ", max(line_digits, 2))
-    padding = if line_digits < 2, do: " "
-
-    " #{placeholder} |\n" <>
-      " #{padding}#{error_line} | #{snippet.content}\n" <>
-      " #{placeholder} | #{String.duplicate(" ", snippet.offset)}^"
+  defp format_location(opts) when is_list(opts) do
+    case opts[:column] do
+      nil -> format_file_line(Keyword.get(opts, :file), Keyword.get(opts, :line), " ")
+      col -> format_file_line_column(Keyword.get(opts, :file), Keyword.get(opts, :line), col, " ")
+    end
   end
 
-  defp format_location(opts) when is_list(opts) do
-    format_file_line(Keyword.get(opts, :file), Keyword.get(opts, :line), " ")
+  @doc false
+  def format_delimiter(delimiter) do
+    if delimiter |> Atom.to_string() |> String.contains?(["\"", "'"]),
+      do: delimiter,
+      else: ~s("#{delimiter}")
+  end
+
+  @doc false
+  def format_snippet(
+        {start_line, _start_column} = start_pos,
+        {end_line, end_column} = end_pos,
+        description,
+        file,
+        lines,
+        start_message,
+        end_message
+      )
+      when start_line < end_line do
+    max_digits = digits(end_line)
+    general_padding = max(2, max_digits) + 1
+    padding = n_spaces(general_padding)
+
+    relevant_lines =
+      if end_line - start_line < 5 do
+        line_range(lines, start_pos, end_pos, padding, max_digits, start_message, end_message)
+      else
+        trimmed_inbetween_lines(
+          lines,
+          start_pos,
+          end_pos,
+          padding,
+          max_digits,
+          start_message,
+          end_message
+        )
+      end
+
+    """
+     #{padding}#{red("error:")} #{pad_message(description, padding)}
+     #{padding}│
+    #{relevant_lines}
+     #{padding}│
+     #{padding}└─ #{Path.relative_to_cwd(file)}:#{end_line}:#{end_column}\
+    """
+  end
+
+  def format_snippet(
+        {start_line, start_column},
+        {end_line, end_column},
+        description,
+        file,
+        lines,
+        start_message,
+        end_message
+      )
+      when start_line == end_line do
+    max_digits = digits(end_line)
+    general_padding = max(2, max_digits) + 1
+    padding = n_spaces(general_padding)
+    formatted_line = [line_padding(end_line, max_digits), to_string(end_line), " │ ", hd(lines)]
+
+    mismatched_closing_line =
+      [
+        n_spaces(start_column - 1),
+        red("│"),
+        format_end_message(end_column - start_column, end_message)
+      ]
+
+    unclosed_delimiter_line =
+      [padding, " │ ", format_start_message(start_column, start_message)]
+
+    below_line = [padding, " │ ", mismatched_closing_line, "\n", unclosed_delimiter_line]
+
+    """
+     #{padding}#{red("error:")} #{pad_message(description, padding)}
+     #{padding}│
+    #{formatted_line}
+    #{below_line}
+     #{padding}│
+     #{padding}└─ #{Path.relative_to_cwd(file)}:#{end_line}:#{end_column}\
+    """
+  end
+
+  defp line_padding(line_number, max_digits) do
+    line_digits = digits(line_number)
+
+    spacing =
+      if line_digits == 1 do
+        max(2, max_digits)
+      else
+        max_digits - line_digits + 1
+      end
+
+    n_spaces(spacing)
+  end
+
+  defp n_spaces(n), do: String.duplicate(" ", n)
+
+  defp digits(number, acc \\ 1)
+  defp digits(number, acc) when number < 10, do: acc
+  defp digits(number, acc), do: digits(div(number, 10), acc + 1)
+
+  defp trimmed_inbetween_lines(
+         lines,
+         {start_line, start_column},
+         {end_line, end_column},
+         padding,
+         max_digits,
+         start_message,
+         end_message
+       ) do
+    start_padding = line_padding(start_line, max_digits)
+    end_padding = line_padding(end_line, max_digits)
+    first_line = hd(lines)
+    last_line = List.last(lines)
+
+    """
+    #{start_padding}#{start_line} │ #{first_line}
+     #{padding}│ #{format_start_message(start_column, start_message)}
+     ...
+    #{end_padding}#{end_line} │ #{last_line}
+     #{padding}│ #{format_end_message(end_column, end_message)}\
+    """
+  end
+
+  defp line_range(
+         lines,
+         {start_line, start_column},
+         {end_line, end_column},
+         padding,
+         max_digits,
+         start_message,
+         end_message
+       ) do
+    Enum.zip_with(lines, start_line..end_line, fn line, line_number ->
+      line_padding = line_padding(line_number, max_digits)
+
+      cond do
+        line_number == start_line ->
+          [
+            line_padding,
+            to_string(line_number),
+            " │ ",
+            line,
+            "\n",
+            padding,
+            " │ ",
+            format_start_message(start_column, start_message)
+          ]
+
+        line_number == end_line ->
+          [
+            line_padding,
+            to_string(line_number),
+            " │ ",
+            line,
+            "\n",
+            padding,
+            " │ ",
+            format_end_message(end_column, end_message)
+          ]
+
+        true ->
+          [line_padding, to_string(line_number), " │ ", line]
+      end
+    end)
+    |> Enum.intersperse("\n")
+  end
+
+  defp format_end_message(end_column, message),
+    do: [
+      n_spaces(end_column - 1),
+      red(message)
+    ]
+
+  defp format_start_message(start_column, message),
+    do: [n_spaces(start_column - 1), red(message)]
+
+  defp pad_message(message, padding), do: String.replace(message, "\n", "\n #{padding}")
+
+  defp red(string) do
+    if IO.ANSI.enabled?() do
+      [IO.ANSI.red(), string, IO.ANSI.reset()]
+    else
+      string
+    end
   end
 end
 
@@ -798,10 +1003,43 @@ end
 # docs.
 
 defmodule RuntimeError do
+  @moduledoc """
+  An exception for a generic runtime error.
+
+  This is the exception that `raise/1` raises when you pass it only a string as
+  a message:
+
+      iex> raise "oops!"
+      ** (RuntimeError) oops!
+
+  You should use this exceptions sparingly, since most of the time it might be
+  better to define your own exceptions specific to your application or library.
+  Sometimes, however, there are situations in which you don't expect a condition to
+  happen, but you want to give a meaningful error message if it does. In those cases,
+  `RuntimeError` can be a good choice.
+
+  ## Fields
+
+  `RuntimeError` exceptions have a single field, `:message` (a `t:String.t/0`),
+  which is public and can be accessed freely when reading or creating `RuntimeError`
+  exceptions.
+  """
+
   defexception message: "runtime error"
 end
 
 defmodule ArgumentError do
+  @moduledoc """
+  An exception raised when an argument to a function is invalid.
+
+  You can raise this exception when you want to signal that an argument to
+  a function is invalid.
+
+  `ArgumentError` exceptions have a single field, `:message` (a `t:String.t/0`),
+  which is public and can be accessed freely when reading or creating `ArgumentError`
+  exceptions.
+  """
+
   defexception message: "argument error"
 
   @impl true
@@ -844,6 +1082,16 @@ defmodule ArgumentError do
 end
 
 defmodule ArithmeticError do
+  @moduledoc """
+  An exception raised on invalid arithmetic operations.
+
+  For example, this exception is raised if you divide by `0`:
+
+      iex> 1 / 0
+      ** (ArithmeticError) bad argument in arithmetic expression: 1 / 0
+
+  """
+
   defexception message: "bad argument in arithmetic expression"
 
   @unary_ops [:+, :-]
@@ -884,10 +1132,105 @@ defmodule ArithmeticError do
 end
 
 defmodule SystemLimitError do
+  @moduledoc """
+  An exception raised when a system limit has been reached.
+
+  For example, this can happen if you try to create an atom that is too large.
+  """
+
   defexception message: "a system limit has been reached"
 end
 
+defmodule MismatchedDelimiterError do
+  @moduledoc """
+  An exception raised when a mismatched delimiter is found when parsing code.
+
+  For example:
+
+    * `[1, 2, 3}`
+    * `fn a -> )`
+
+  The following fields of this exceptions are public and can be accessed freely:
+
+    * `:file` (`t:Path.t/0` or `nil`) - the file where the error occurred, or `nil` if
+      the error occurred in code that did not come from a file
+    * `:line` - the line for the opening delimiter
+    * `:column` - the column for the opening delimiter
+    * `:end_line` - the line for the mismatched closing delimiter
+    * `:end_column` - the column for the mismatched closing delimiter
+    * `:opening_delimiter` - an atom representing the opening delimiter
+    * `:closing_delimiter` - an atom representing the mismatched closing delimiter
+    * `:expected_delimiter` - an atom representing the closing delimiter
+    * `:description` - a description of the mismatched delimiter error
+
+  """
+
+  defexception [
+    :file,
+    :line,
+    :column,
+    :end_line,
+    :end_column,
+    :opening_delimiter,
+    :closing_delimiter,
+    :expected_delimiter,
+    :snippet,
+    description: "mismatched delimiter error"
+  ]
+
+  @impl true
+  def message(%{
+        line: start_line,
+        column: start_column,
+        end_line: end_line,
+        end_column: end_column,
+        description: description,
+        expected_delimiter: expected_delimiter,
+        file: file,
+        snippet: snippet
+      }) do
+    start_pos = {start_line, start_column}
+    end_pos = {end_line, end_column}
+    lines = String.split(snippet, "\n")
+    expected_delimiter = Exception.format_delimiter(expected_delimiter)
+
+    start_message = "└ unclosed delimiter"
+    end_message = ~s/└ mismatched closing delimiter (expected #{expected_delimiter})/
+
+    snippet =
+      Exception.format_snippet(
+        start_pos,
+        end_pos,
+        description,
+        file,
+        lines,
+        start_message,
+        end_message
+      )
+
+    format_message(file, end_line, end_column, snippet)
+  end
+
+  defp format_message(file, line, column, message) do
+    location = Exception.format_file_line_column(Path.relative_to_cwd(file), line, column)
+    "mismatched delimiter found on " <> location <> "\n" <> message
+  end
+end
+
 defmodule SyntaxError do
+  @moduledoc """
+  An exception raised when there's a syntax error when parsing code.
+
+  The following fields of this exceptions are public and can be accessed freely:
+
+    * `:file` (`t:Path.t/0` or `nil`) - the file where the error occurred, or `nil` if
+      the error occurred in code that did not come from a file
+    * `:line` - the line where the error occurred
+    * `:column` - the column where the error occurred
+    * `:description` - a description of the syntax error
+
+  """
+
   defexception [:file, :line, :column, :snippet, description: "syntax error"]
 
   @impl true
@@ -899,8 +1242,10 @@ defmodule SyntaxError do
         snippet: snippet
       })
       when not is_nil(snippet) and not is_nil(column) do
-    Exception.format_file_line_column(Path.relative_to_cwd(file), line, column) <>
-      " " <> description <> "\n" <> Exception.format_snippet(snippet, line)
+    snippet =
+      :elixir_errors.format_snippet({line, column}, file, description, snippet, :error, [], nil)
+
+    format_message(file, line, column, snippet)
   end
 
   @impl true
@@ -910,25 +1255,87 @@ defmodule SyntaxError do
         column: column,
         description: description
       }) do
-    Exception.format_file_line_column(Path.relative_to_cwd(file), line, column) <>
-      " " <> description
+    snippet =
+      :elixir_errors.format_snippet({line, column}, file, description, nil, :error, [], nil)
+
+    padded = "   " <> String.replace(snippet, "\n", "\n   ")
+    format_message(file, line, column, padded)
+  end
+
+  defp format_message(file, line, column, message) do
+    location = Exception.format_file_line_column(Path.relative_to_cwd(file), line, column)
+    "invalid syntax found on " <> location <> "\n" <> message
   end
 end
 
 defmodule TokenMissingError do
-  defexception [:file, :line, :snippet, :column, description: "expression is incomplete"]
+  @moduledoc """
+  An exception raised when a token is missing when parsing code.
+
+  The following fields of this exceptions are public and can be accessed freely:
+
+    * `:file` (`t:Path.t/0` or `nil`) - the file where the error occurred, or `nil` if
+      the error occurred in code that did not come from a file
+    * `:line` - the line for the opening delimiter
+    * `:column` - the column for the opening delimiter
+    * `:end_line` - the line for the end of the string
+    * `:end_column` - the column for the end of the string
+    * `:opening_delimiter` - an atom representing the opening delimiter
+    * `:expected_delimiter` - an atom representing the expected delimiter
+    * `:description` - a description of the missing token error
+  """
+
+  defexception [
+    :file,
+    :line,
+    :column,
+    :end_line,
+    :end_column,
+    :snippet,
+    :opening_delimiter,
+    :expected_delimiter,
+    description: "expression is incomplete"
+  ]
 
   @impl true
   def message(%{
         file: file,
         line: line,
         column: column,
+        end_line: end_line,
         description: description,
+        expected_delimiter: expected_delimiter,
         snippet: snippet
       })
-      when not is_nil(snippet) and not is_nil(column) do
-    Exception.format_file_line_column(Path.relative_to_cwd(file), line, column) <>
-      " " <> description <> "\n" <> Exception.format_snippet(snippet, line)
+      when not is_nil(snippet) and not is_nil(column) and not is_nil(end_line) do
+    {trimmed, [last_line | _] = reversed_lines} =
+      snippet
+      |> String.split("\n")
+      |> Enum.reverse()
+      |> Enum.split_while(&(&1 == ""))
+
+    end_line = end_line - length(trimmed)
+    end_column = String.length(last_line) + 1
+
+    start_pos = {line, column}
+    end_pos = {end_line, end_column}
+    expected_delimiter = Exception.format_delimiter(expected_delimiter)
+
+    start_message = ~s/└ unclosed delimiter/
+    end_message = ~s/└ missing closing delimiter (expected #{expected_delimiter})/
+
+    snippet =
+      Exception.format_snippet(
+        start_pos,
+        end_pos,
+        description,
+        file,
+        Enum.reverse(reversed_lines),
+        start_message,
+        end_message
+      )
+
+    format_message(file, end_line, end_column, snippet)
   end
 
   @impl true
@@ -936,14 +1343,33 @@ defmodule TokenMissingError do
         file: file,
         line: line,
         column: column,
+        snippet: snippet,
         description: description
       }) do
-    Exception.format_file_line_column(file && Path.relative_to_cwd(file), line, column) <>
-      " " <> description
+    snippet =
+      :elixir_errors.format_snippet({line, column}, file, description, snippet, :error, [], nil)
+
+    format_message(file, line, column, snippet)
+  end
+
+  defp format_message(file, line, column, message) do
+    location = Exception.format_file_line_column(Path.relative_to_cwd(file), line, column)
+    "token missing on " <> location <> "\n" <> message
   end
 end
 
 defmodule CompileError do
+  @moduledoc """
+  An exception raised when there's an error when compiling code.
+
+  The following fields of this exceptions are public and can be accessed freely:
+
+    * `:file` (`t:Path.t/0` or `nil`) - the file where the error occurred, or `nil` if
+      the error occurred in code that did not come from a file
+    * `:line` (`t:non_neg_integer/0`) - the line where the error occurred
+
+  """
+
   defexception [:file, :line, description: "compile error"]
 
   @impl true
@@ -956,6 +1382,17 @@ defmodule CompileError do
 end
 
 defmodule Kernel.TypespecError do
+  @moduledoc """
+  An exception raised when there's an error in a typespec.
+
+  The following fields of this exceptions are public and can be accessed freely:
+
+    * `:file` (`t:Path.t/0` or `nil`) - the file where the error occurred, or `nil` if
+      the error occurred in code that did not come from a file
+    * `:line` (`t:non_neg_integer/0`) - the line where the error occurred
+
+  """
+
   defexception [:file, :line, :description]
 
   @impl true
@@ -990,6 +1427,10 @@ defmodule BadStructError do
 end
 
 defmodule BadMapError do
+  @moduledoc """
+  An exception raised when something expected a map, but received something else.
+  """
+
   defexception [:term]
 
   @impl true
@@ -999,6 +1440,10 @@ defmodule BadMapError do
 end
 
 defmodule BadBooleanError do
+  @moduledoc """
+  An exception raised when an operator expected a boolean, but received something else.
+  """
+
   defexception [:term, :operator]
 
   @impl true
@@ -1008,6 +1453,19 @@ defmodule BadBooleanError do
 end
 
 defmodule MatchError do
+  @moduledoc """
+  An exception raised when a pattern match (`=/2`) fails.
+
+  The following fields of this exception are public and can be accessed freely:
+
+    * `:term` (`t:term/0`) - the term that did not match the pattern
+
+  For example, this exception gets raised for code like this:
+
+      [_ | _] = []
+
+  """
+
   defexception [:term]
 
   @impl true
@@ -1017,6 +1475,23 @@ defmodule MatchError do
 end
 
 defmodule CaseClauseError do
+  @moduledoc """
+  An exception raised when a term in a `case/2` expression
+  does not match any of the defined `->` clauses.
+
+  The following fields of this exception are public and can be accessed freely:
+
+    * `:term` (`t:term/0`) - the term that did not match any of the clauses
+
+  For example, this exception gets raised for a `case/2` like the following:
+
+      case System.unique_integer() do
+        bin when is_binary(bin) -> :oops
+        :ok -> :neither_this_one
+      end
+
+  """
+
   defexception [:term]
 
   @impl true
@@ -1026,6 +1501,27 @@ defmodule CaseClauseError do
 end
 
 defmodule WithClauseError do
+  @moduledoc """
+  An exception raised when a term in a `with/1` expression
+  does not match any of the defined `->` clauses in its `else`.
+
+  The following fields of this exception are public and can be accessed freely:
+
+    * `:term` (`t:term/0`) - the term that did not match any of the clauses
+
+  For example, this exception gets raised for a `with/1` like the following, because
+  the `{:ok, 2}` term does not match the `:error` or `{:error, _}` clauses in the
+  `else`:
+
+      with {:ok, 1} <- {:ok, 2} do
+        :woah
+      else
+        :error -> :error
+        {:error, _} -> :error
+      end
+
+  """
+
   defexception [:term]
 
   @impl true
@@ -1035,6 +1531,18 @@ defmodule WithClauseError do
 end
 
 defmodule CondClauseError do
+  @moduledoc """
+  An exception raised when no clauses in a `cond/1` expression evaluate to a truthy value.
+
+  For example, this exception gets raised for a `cond/1` like the following:
+
+      cond do
+        1 + 1 == 3 -> :woah
+        nil -> "yeah this won't happen
+      end
+
+  """
+
   defexception []
 
   @impl true
@@ -1044,6 +1552,15 @@ defmodule CondClauseError do
 end
 
 defmodule TryClauseError do
+  @moduledoc """
+  An exception raised when a term in a `try/1` expression
+  does not match any of the defined `->` clauses in its `else`.
+
+  The following fields of this exception are public and can be accessed freely:
+
+    * `:term` (`t:term/0`) - the term that did not match any of the clauses
+
+  """
   defexception [:term]
 
   @impl true
@@ -1053,6 +1570,10 @@ defmodule TryClauseError do
 end
 
 defmodule BadArityError do
+  @moduledoc """
+  An exception raised when a function is called with the wrong number of arguments.
+  """
+
   defexception [:function, :args]
 
   @impl true
@@ -1070,12 +1591,33 @@ defmodule BadArityError do
 end
 
 defmodule UndefinedFunctionError do
+  @moduledoc """
+  An exception raised when a function is invoked that is not defined.
+
+  The following fields of this exception are public and can be accessed freely:
+
+    * `:module` (`t:module/0`) - the module name
+    * `:function` (`t:atom/0`) - the function name
+    * `:arity` (`t:non_neg_integer/0`) - the arity of the function
+
+  For example, if you try to call `MyMod.non_existing_fun("hello", 1)`,
+  the error would look like:
+
+      %UndefinedFunctionError{
+        module: MyMod,
+        function: :non_existing_fun,
+        arity: 2,
+        # Other private fields...
+      }
+
+  """
+
   defexception [:module, :function, :arity, :reason, :message]
 
   @impl true
   def message(%{message: nil} = exception) do
     %{reason: reason, module: module, function: function, arity: arity} = exception
-    {message, _loaded?} = message(reason, module, function, arity)
+    {message, _hint_type} = message(reason, module, function, arity)
     message
   end
 
@@ -1086,11 +1628,11 @@ defmodule UndefinedFunctionError do
   defp message(nil, module, function, arity) do
     cond do
       is_nil(function) or is_nil(arity) ->
-        {"undefined function", false}
+        {"undefined function", :suggest_module}
 
       is_nil(module) ->
         formatted_fun = Exception.format_mfa(module, function, arity)
-        {"function #{formatted_fun} is undefined", false}
+        {"function #{formatted_fun} is undefined", :suggest_module}
 
       function_exported?(module, :module_info, 0) ->
         message(:"function not exported", module, function, arity)
@@ -1102,39 +1644,83 @@ defmodule UndefinedFunctionError do
 
   defp message(:"module could not be loaded", module, function, arity) do
     formatted_fun = Exception.format_mfa(module, function, arity)
-    {"function #{formatted_fun} is undefined (module #{inspect(module)} is not available)", false}
+
+    {"function #{formatted_fun} is undefined (module #{inspect(module)} is not available)",
+     :suggest_module}
   end
 
   defp message(:"function not exported", module, function, arity) do
     formatted_fun = Exception.format_mfa(module, function, arity)
-    {"function #{formatted_fun} is undefined or private", true}
+    {"function #{formatted_fun} is undefined or private", :suggest_function}
+  end
+
+  defp message(:"undefined local", nil, function, arity) do
+    {"function #{function}/#{arity} is undefined (there is no such import)", :no_hint}
   end
 
   defp message(reason, module, function, arity) do
     formatted_fun = Exception.format_mfa(module, function, arity)
-    {"function #{formatted_fun} is undefined (#{reason})", false}
+    {"function #{formatted_fun} is undefined (#{reason})", :suggest_module}
   end
 
   @impl true
   def blame(exception, stacktrace) do
     %{reason: reason, module: module, function: function, arity: arity} = exception
-    {message, loaded?} = message(reason, module, function, arity)
-    message = message <> hint(module, function, arity, loaded?)
+    {message, hint_type} = message(reason, module, function, arity)
+    message = message <> hint(module, function, arity, hint_type)
     {%{exception | message: message}, stacktrace}
   end
 
-  defp hint(nil, _function, 0, _loaded?) do
+  defp hint(_, _, _, :no_hint), do: ""
+
+  defp hint(nil, _function, 0, _hint_type) do
     ". If you are using the dot syntax, such as module.function(), " <>
       "make sure the left-hand side of the dot is a module atom"
   end
 
-  defp hint(module, function, arity, true) do
+  defp hint(module, function, arity, :suggest_function) do
     behaviour_hint(module, function, arity) <>
       hint_for_loaded_module(module, function, arity, nil)
   end
 
-  defp hint(_module, _function, _arity, _loaded?) do
-    ""
+  @max_suggestions 5
+  defp hint(module, function, arity, :suggest_module) do
+    downcased_module = downcase_module_name(module)
+    stripped_module = module |> Atom.to_string() |> String.replace_leading("Elixir.", "")
+
+    candidates =
+      for {name, _, _} = candidate <- :code.all_available(),
+          downcase_module_name(name) == downcased_module or
+            String.ends_with?(List.to_string(name), stripped_module),
+          {:module, module} <- [load_module(candidate)],
+          function_exported?(module, function, arity),
+          do: module
+
+    if candidates != [] do
+      suggestions =
+        candidates
+        |> Enum.take(@max_suggestions)
+        |> Enum.sort(:asc)
+        |> Enum.map(fn module ->
+          ["\n      * ", Exception.format_mfa(module, function, arity)]
+        end)
+
+      IO.iodata_to_binary([". Did you mean:\n", suggestions, "\n"])
+    else
+      ""
+    end
+  end
+
+  defp load_module({name, _path, _loaded?}) do
+    name
+    |> List.to_atom()
+    |> Code.ensure_loaded()
+  end
+
+  defp downcase_module_name(module) do
+    module
+    |> to_string()
+    |> String.downcase(:ascii)
   end
 
   @doc false
@@ -1248,6 +1834,27 @@ defmodule UndefinedFunctionError do
 end
 
 defmodule FunctionClauseError do
+  @moduledoc """
+  An exception raised when a function call doesn't match any defined clause.
+
+  The following fields of this exception are public and can be accessed freely:
+
+    * `:module` (`t:module/0`) - the module name
+    * `:function` (`t:atom/0`) - the function name
+    * `:arity` (`t:non_neg_integer/0`) - the arity of the function
+
+  For example, if you try to call a function such as `URI.parse/1` with something
+  other than a string, the error would look like:
+
+      %FunctionClauseError{
+        module: URI,
+        function: :parse,
+        arity: 1,
+        # Other private fields...
+      }
+
+  """
+
   defexception [:module, :function, :arity, :kind, :args, :clauses]
 
   @clause_limit 10
@@ -1362,6 +1969,16 @@ defmodule FunctionClauseError do
 end
 
 defmodule Code.LoadError do
+  @moduledoc """
+  An exception raised when a file cannot be loaded.
+
+  The following fields of this exception are public and can be accessed freely:
+
+    * `:file` (`t:String.t/0`) - the file name
+    * `:reason` (`t:term/0`) - the reason why the file could not be loaded
+
+  """
+
   defexception [:file, :message, :reason]
 
   def exception(opts) do
@@ -1373,6 +1990,28 @@ defmodule Code.LoadError do
 end
 
 defmodule Protocol.UndefinedError do
+  @moduledoc """
+  An exception raised when a protocol is not implemented for a given value.
+
+  The following fields of this exception are public and can be accessed freely:
+
+    * `:protocol` (`t:module/0`) - the protocol that is not implemented
+    * `:value` (`t:term/0`) - the value that does not implement the protocol
+
+  For example, this code:
+
+      Enum.at("A string!", 0)
+
+  would raise the following exception:
+
+      %Protocol.UndefinedError{
+        protocol: Enumerable,
+        value: "A string!",
+        # ...
+      }
+
+  """
+
   defexception [:protocol, :value, description: ""]
 
   @impl true
@@ -1413,6 +2052,19 @@ defmodule Protocol.UndefinedError do
 end
 
 defmodule KeyError do
+  @moduledoc """
+  An exception raised when a key is not found in a data structure.
+
+  For example, this is raised by `Map.fetch!/2` when the given key
+  cannot be found in the given map.
+
+  The following fields of this exception are public and can be accessed freely:
+
+    * `:term` (`t:term/0`) - the data structure that was searched
+    * `:key` (`t:term/0`) - the key that was not found
+
+  """
+
   defexception [:key, :term, :message]
 
   @impl true
@@ -1509,14 +2161,38 @@ defmodule UnicodeConversionError do
 end
 
 defmodule Enum.OutOfBoundsError do
+  @moduledoc """
+  An exception that is raised when a function expects an enumerable to have
+  a certain size but finds that it is too small.
+
+  For example, this is raised by `Access.at!/1`.
+  """
+
   defexception message: "out of bounds error"
 end
 
 defmodule Enum.EmptyError do
+  @moduledoc """
+  An exception that is raised when something expects a non-empty enumerable
+  but finds an empty one.
+
+  For example, this is raised by `Enum.min/3`.
+  """
+
   defexception message: "empty error"
 end
 
 defmodule File.Error do
+  @moduledoc """
+  An exception that is raised when a file operation fails.
+
+  The following fields of this exception are public and can be accessed freely:
+
+    * `:path` (`t:Path.t/0`) - the path of the file that caused the error
+    * `:reason` (`t:File.posix/0`) - the reason for the error
+
+  """
+
   defexception [:reason, :path, action: ""]
 
   @impl true
@@ -1535,6 +2211,17 @@ defmodule File.Error do
 end
 
 defmodule File.CopyError do
+  @moduledoc """
+  An exception that is raised when copying a file fails.
+
+  The following fields of this exception are public and can be accessed freely:
+
+    * `:source` (`t:Path.t/0`) - the source path
+    * `:destination` (`t:Path.t/0`) - the destination path
+    * `:reason` (`t:File.posix/0`) - the reason why the file could not be copied
+
+  """
+
   defexception [:reason, :source, :destination, on: "", action: ""]
 
   @impl true
@@ -1553,6 +2240,17 @@ defmodule File.CopyError do
 end
 
 defmodule File.RenameError do
+  @moduledoc """
+  An exception that is raised when renaming a file fails.
+
+  The following fields of this exception are public and can be accessed freely:
+
+    * `:source` (`t:Path.t/0`) - the source path
+    * `:destination` (`t:Path.t/0`) - the destination path
+    * `:reason` (`t:File.posix/0`) - the reason why the file could not be renamed
+
+  """
+
   defexception [:reason, :source, :destination, on: "", action: ""]
 
   @impl true
@@ -1571,6 +2269,17 @@ defmodule File.RenameError do
 end
 
 defmodule File.LinkError do
+  @moduledoc """
+  An exception that is raised when linking a file fails.
+
+  The following fields of this exception are public and can be accessed freely:
+
+    * `:existing` (`t:Path.t/0`) - the existing file to link
+    * `:new` (`t:Path.t/0`) - the link destination
+    * `:reason` (`t:File.posix/0`) - the reason why the file could not be linked
+
+  """
+
   defexception [:reason, :existing, :new, action: ""]
 
   @impl true
