@@ -8,6 +8,13 @@ defmodule GenServer do
   will have a standard set of interface functions and include functionality for
   tracing and error reporting. It will also fit into a supervision tree.
 
+  ```mermaid
+  graph BT
+      C(Client #3) ~~~ B(Client #2) ~~~ A(Client #1)
+      A & B & C -->|request| GenServer
+      GenServer -.->|reply| A & B & C
+  ```
+
   ## Example
 
   The GenServer behaviour abstracts the common client-server interaction.
@@ -136,11 +143,44 @@ defmodule GenServer do
         end
       end
 
-
   In practice, it is common to have both server and client functions in
   the same module. If the server and/or client implementations are growing
   complex, you may want to have them in different modules.
 
+  The following diagram summarizes the interactions between client and server.
+  Both Client and Server are processes and communication happens via messages
+  (continuous line). The Server <-> Module interaction happens when the
+  GenServer process calls your code (dotted lines):
+
+  ```mermaid
+  sequenceDiagram
+      participant C as Client (Process)
+      participant S as Server (Process)
+      participant M as Module (Code)
+
+      note right of C: Typically started by a supervisor
+      C->>+S: GenServer.start_link(module, arg, options)
+      S-->>+M: init(arg)
+      M-->>-S: {:ok, state} | :ignore | {:error, reason}
+      S->>-C: {:ok, pid} | :ignore | {:error, reason}
+
+      note right of C: call is synchronous
+      C->>+S: GenServer.call(pid, message)
+      S-->>+M: handle_call(message, from, state)
+      M-->>-S: {:reply, reply, state} | {:stop, reason, reply, state}
+      S->>-C: reply
+
+      note right of C: cast is asynchronous
+      C-)S: GenServer.cast(pid, message)
+      S-->>+M: handle_cast(message, state)
+      M-->>-S: {:noreply, state} | {:stop, reason, state}
+
+      note right of C: send is asynchronous
+      C-)S: Kernel.send(pid, message)
+      S-->>+M: handle_info(message, state)
+      M-->>-S: {:noreply, state} | {:stop, reason, state}
+  ```
+
   ## How to supervise
 
   A `GenServer` is most commonly started under a supervision tree.
@@ -431,7 +471,7 @@ defmodule GenServer do
   guide provides a tutorial-like introduction. The documentation and links
   in Erlang can also provide extra insight.
 
-    * [GenServer - Elixir's Getting Started Guide](https://elixir-lang.org/getting-started/mix-otp/genserver.html)
+    * [GenServer - Elixir's Getting Started Guide](genservers.md)
     * [`:gen_server` module documentation](`:gen_server`)
     * [gen_server Behaviour - OTP Design Principles](https://www.erlang.org/doc/design_principles/gen_server_concepts.html)
     * [Clients and Servers - Learn You Some Erlang for Great Good!](http://learnyousomeerlang.com/clients-and-servers)
