@@ -268,9 +268,11 @@ defmodule IO do
   to stdio with this function will likely result in the wrong data
   being sent down the wire.
   """
-  @spec binwrite(device, iodata) :: :ok | {:error, term}
+  @spec binwrite(device, iodata) :: :ok
   def binwrite(device \\ :stdio, iodata) when is_iodata(iodata) do
-    :file.write(map_dev(device), iodata)
+    with {:error, reason} <- :file.write(map_dev(device), iodata) do
+      :erlang.error(reason)
+    end
   end
 
   @doc """
@@ -306,17 +308,20 @@ defmodule IO do
       entry from the compilation environment will be used
 
     * a keyword list with at least the `:file` option representing
-      a single stacktrace entry (since v1.14.0). The `:line`, `:module`,
-      `:function` options are also supported
+      a single stacktrace entry (since v1.14.0). The `:line`, `:column`,
+      `:module`, and `:function` options are also supported
 
-  This function also notifies the compiler a warning was printed
-  (in case --warnings-as-errors was enabled). It returns `:ok`
-  if it succeeds.
+  This function notifies the compiler a warning was printed
+  and emits a compiler diagnostic (`t:Code.diagnostic/1`).
+  The diagnostic will include precise file and location information
+  if a `Macro.Env` is given or those values have been passed as
+  keyword list, but not for stacktraces, as they are often imprecise.
+
+  It returns `:ok` if it succeeds.
 
   ## Examples
 
-      stacktrace = [{MyApp, :main, 1, [file: 'my_app.ex', line: 4]}]
-      IO.warn("variable bar is unused", stacktrace)
+      IO.warn("variable bar is unused", module: MyApp, function: {:main, 1}, line: 4, file: "my_app.ex")
       #=> warning: variable bar is unused
       #=>   my_app.ex:4: MyApp.main/1
 
@@ -325,37 +330,46 @@ defmodule IO do
           :ok
   def warn(message, stacktrace_info)
 
-  def warn(message, %Macro.Env{} = env) do
-    warn(message, Macro.Env.stacktrace(env))
-  end
+  def warn(message, %Macro.Env{line: line, file: file} = env) do
+    message = to_chardata(message)
 
-  def warn(message, []) do
-    :elixir_errors.emit_diagnostic(:warning, 0, nil, to_chardata(message), [])
+    :elixir_errors.emit_diagnostic(:warning, line, file, message, Macro.Env.stacktrace(env),
+      read_snippet: true
+    )
   end
 
   def warn(message, [{_, _} | _] = keyword) do
     if file = keyword[:file] do
-      warn(
-        message,
-        %{
+      line = keyword[:line]
+      column = keyword[:column]
+      position = if line && column, do: {line, column}, else: line
+      message = to_chardata(message)
+
+      stacktrace =
+        Macro.Env.stacktrace(%{
           __ENV__
           | module: keyword[:module],
             function: keyword[:function],
-            line: keyword[:line],
+            line: line,
             file: file
-        }
+        })
+
+      :elixir_errors.emit_diagnostic(:warning, position, file, message, stacktrace,
+        read_snippet: true
       )
     else
       warn(message, [])
     end
   end
 
-  def warn(message, [{_, _, _, opts} | _] = stacktrace) do
+  def warn(message, []) do
     message = to_chardata(message)
-    line = opts[:line]
-    file = opts[:file]
-    file = file && List.to_string(file)
-    :elixir_errors.emit_diagnostic(:warning, line || 0, file, message, stacktrace)
+    :elixir_errors.emit_diagnostic(:warning, 0, nil, message, [], read_snippet: false)
+  end
+
+  def warn(message, [{_, _, _, _} | _] = stacktrace) do
+    message = to_chardata(message)
+    :elixir_errors.emit_diagnostic(:warning, 0, nil, message, stacktrace, read_snippet: false)
   end
 
   @doc false
@@ -563,6 +577,7 @@ defmodule IO do
 
   """
   @doc since: "1.12.0"
+  @spec stream() :: Enumerable.t(String.t())
   def stream, do: stream(:stdio, :line)
 
   @doc """
@@ -616,6 +631,7 @@ defmodule IO do
 
   """
   @doc since: "1.12.0"
+  @spec binstream() :: Enumerable.t(binary)
   def binstream, do: binstream(:stdio, :line)
 
   @doc """
