@@ -187,20 +187,23 @@ defmodule Kernel.SpecialForms do
       iex> <<0, "foo">>
       <<0, 102, 111, 111>>
 
-  Binaries need to be explicitly tagged as `binary`:
+  You can use one of `utf8` (the default), `utf16`, and `utf32` to
+  control how the string is encoded:
+
+      iex> <<"foo"::utf16>>
+      <<0, 102, 0, 111, 0, 111>>
+
+  Which is equivalent to writing:
+
+      iex> <<?f::utf16, ?o::utf16, ?o::utf16>>
+      <<0, 102, 0, 111, 0, 111>>
+
+  At runtime, binaries need to be explicitly tagged as `binary`:
 
       iex> rest = "oo"
       iex> <<102, rest::binary>>
       "foo"
 
-  The `utf8`, `utf16`, and `utf32` types are for Unicode code points. They
-  can also be applied to literal strings and charlists:
-
-      iex> <<"foo"::utf16>>
-      <<0, 102, 0, 111, 0, 111>>
-      iex> <<"foo"::utf32>>
-      <<0, 0, 0, 102, 0, 0, 0, 111, 0, 0, 0, 111>>
-
   Otherwise we get an `ArgumentError` when constructing the binary:
 
       rest = "oo"
@@ -294,7 +297,7 @@ defmodule Kernel.SpecialForms do
   `unsigned` (default) | `integer`
   `little`             | `integer`, `float`, `utf16`, `utf32`
   `big` (default)      | `integer`, `float`, `utf16`, `utf32`
-  `native`             | `integer`, `utf16`, `utf32`
+  `native`             | `integer`, `float`, `utf16`, `utf32`
 
   ### Sign
 
@@ -346,7 +349,7 @@ defmodule Kernel.SpecialForms do
 
   Or as a part of function definitions to pattern match:
 
-      defmodule ImageTyper do
+      defmodule ImageType do
         @png_signature <<137::size(8), 80::size(8), 78::size(8), 71::size(8),
                          13::size(8), 10::size(8), 26::size(8), 10::size(8)>>
         @jpg_signature <<255::size(8), 216::size(8)>>
@@ -1023,9 +1026,10 @@ defmodule Kernel.SpecialForms do
         end
       end
 
+      require Hygiene
       Hygiene.write()
       Hygiene.read()
-      ** (RuntimeError) undefined variable a or undefined function a/0
+      ** (CompileError) undefined variable "a" (context Hygiene)
 
   For such, you can explicitly pass the current module scope as
   argument:
@@ -1044,6 +1048,7 @@ defmodule Kernel.SpecialForms do
         end
       end
 
+      require Hygiene
       ContextHygiene.write()
       ContextHygiene.read()
       #=> 1
@@ -1540,7 +1545,7 @@ defmodule Kernel.SpecialForms do
   area, as `{:ok, area}` or return `:error`. We could implement
   this function as:
 
-      def area(map) do
+      def area(opts) do
         case Map.fetch(opts, :width) do
           {:ok, width} ->
             case Map.fetch(opts, :height) do
@@ -1559,7 +1564,7 @@ defmodule Kernel.SpecialForms do
   While the code above works, it is quite verbose. Using `with`,
   we could rewrite it as:
 
-      def area(map) do
+      def area(opts) do
         with {:ok, width} <- Map.fetch(opts, :width),
              {:ok, height} <- Map.fetch(opts, :height) do
           {:ok, width * height}
