@@ -155,8 +155,7 @@ defmodule Module.ParallelChecker do
 
     case :erlang.get(:elixir_code_diagnostics) do
       :undefined -> :ok
-      {tail, true} -> :erlang.put(:elixir_code_diagnostics, {diagnostics ++ tail, true})
-      {tail, false} -> :erlang.put(:elixir_code_diagnostics, {diagnostics ++ tail, false})
+      {tail, log?} -> :erlang.put(:elixir_code_diagnostics, {diagnostics ++ tail, log?})
     end
 
     diagnostics
@@ -168,8 +167,9 @@ defmodule Module.ParallelChecker do
 
   defp collect_results(count, diagnostics) do
     receive do
-      {:diagnostic, diagnostic} ->
-        diagnostic = format_diagnostic_file(diagnostic)
+      {:diagnostic, %{file: file} = diagnostic, read_snippet} ->
+        :elixir_errors.print_diagnostic(diagnostic, read_snippet)
+        diagnostic = %{diagnostic | file: file && Path.absname(file)}
         collect_results(count, [diagnostic | diagnostics])
 
       {__MODULE__, _module, new_diagnostics} ->
@@ -288,11 +288,6 @@ defmodule Module.ParallelChecker do
     end
   end
 
-  @doc false
-  def format_diagnostic_file(%{file: file} = diagnostic) do
-    %{diagnostic | file: file && Path.absname(file)}
-  end
-
   ## Warning helpers
 
   defp group_warnings(warnings) do
@@ -309,79 +304,88 @@ defmodule Module.ParallelChecker do
     Enum.flat_map(warnings, fn {module, warning, locations} ->
       message = module.format_warning(warning)
       diagnostics = Enum.map(locations, &to_diagnostic(message, &1))
-      log? and :elixir_errors.print_warning([message, ?\n, format_stacktraces(diagnostics)])
+      log? and print_warning(message, diagnostics)
       diagnostics
     end)
   end
 
-  defp format_stacktraces([diagnostic]) do
-    format_diagnostic_stacktrace(diagnostic)
+  defp print_warning(message, [diagnostic]) do
+    :elixir_errors.print_warning(message, diagnostic)
   end
 
-  defp format_stacktraces(diagnostics) do
-    [
-      "Invalid call found at #{length(diagnostics)} locations:\n",
-      Enum.map(diagnostics, &format_diagnostic_stacktrace/1)
-    ]
+  defp print_warning(message, grouped_warnings) do
+    :elixir_errors.print_warning_group(message, grouped_warnings)
   end
 
-  defp format_diagnostic_stacktrace(%{stacktrace: [stacktrace]}) do
-    ["  ", Exception.format_stacktrace_entry(stacktrace), ?\n]
-  end
-
-  defp to_diagnostic(message, {file, line, mfa}) do
+  defp to_diagnostic(message, {file, position, mfa}) when is_list(position) do
     %{
       severity: :warning,
+      source: file,
       file: file,
-      position: line,
+      position: position_to_tuple(position),
       message: IO.iodata_to_binary(message),
-      stacktrace: [to_stacktrace(file, line, mfa)]
+      stacktrace: [to_stacktrace(file, position, mfa)],
+      span: nil
     }
   end
 
-  defp to_stacktrace(file, line, {module, fun, arity}),
-    do: {module, fun, arity, location(file, line)}
+  defp position_to_tuple(position) do
+    case position[:column] do
+      nil -> position[:line] || 0
+      col -> {position[:line], col}
+    end
+  end
 
-  defp to_stacktrace(file, line, nil),
-    do: {:elixir_compiler, :__FILE__, 1, location(file, line)}
+  defp to_stacktrace(file, pos, {module, fun, arity}),
+    do: {module, fun, arity, location(file, pos)}
 
-  defp to_stacktrace(file, line, module),
-    do: {module, :__MODULE__, 0, location(file, line)}
+  defp to_stacktrace(file, pos, nil),
+    do: {:elixir_compiler, :__FILE__, 1, location(file, pos)}
 
-  defp location(file, line) do
-    [file: String.to_charlist(Path.relative_to_cwd(file)), line: line]
+  defp to_stacktrace(file, pos, module),
+    do: {module, :__MODULE__, 0, location(file, pos)}
+
+  defp location(file, position) do
+    [{:file, String.to_charlist(Path.relative_to_cwd(file))} | position]
   end
 
   ## Cache
 
   defp cache_module({server, ets}, module) do
     if lock(server, module) do
-      cache_from_chunk(ets, module) || cache_from_info(ets, module)
+      object_code = :code.get_object_code(module)
+
+      # The chunk has more information, so that's our preference
+      with {^module, binary, _filename} <- object_code,
+           {:ok, {^module, [{~c"ExCk", chunk}]}} <- :beam_lib.chunks(binary, [~c"ExCk"]),
+           {:elixir_checker_v1, contents} <- :erlang.binary_to_term(chunk) do
+        cache_chunk(ets, module, contents.exports)
+      else
+        _ ->
+          # Otherwise, if the module is loaded, use its info
+          case :erlang.module_loaded(module) do
+            true ->
+              {mode, exports} = info_exports(module)
+              deprecated = info_deprecated(module)
+              cache_info(ets, module, exports, deprecated, mode)
+
+            false ->
+              # Or load exports from chunk
+              with {^module, binary, _filename} <- object_code,
+                   {:ok, {^module, [exports: exports]}} <- :beam_lib.chunks(binary, [:exports]) do
+                exports = Map.new(Enum.map(exports, &{&1, :def}))
+                cache_info(ets, module, exports, %{}, :erlang)
+              else
+                _ ->
+                  :ets.insert(ets, {{:cached, module}, false})
+              end
+          end
+      end
+
       unlock(server, module)
     end
   end
 
-  defp cache_from_chunk(ets, module) do
-    with {^module, binary, _filename} <- :code.get_object_code(module),
-         {:ok, {^module, [{~c"ExCk", chunk}]}} <- :beam_lib.chunks(binary, [~c"ExCk"]),
-         {:elixir_checker_v1, contents} <- :erlang.binary_to_term(chunk) do
-      cache_chunk(ets, module, contents.exports)
-      true
-    else
-      _ -> false
-    end
-  end
-
-  defp cache_from_info(ets, module) do
-    if Code.ensure_loaded?(module) do
-      {mode, exports} = info_exports(module)
-      deprecated = info_deprecated(module)
-      cache_info(ets, module, exports, deprecated, mode)
-    else
-      :ets.insert(ets, {{:cached, module}, false})
-    end
-  end
-
   defp info_exports(module) do
     map =
       Map.new(
