@@ -26,13 +26,15 @@ defmodule Module.Types do
 
             error =
               Error.exception("""
-              found error while checking types for #{Exception.format_mfa(module, fun, arity)}
+              found error while checking types for #{Exception.format_mfa(module, fun, arity)}:
+
+              #{Exception.format_banner(:error, e, __STACKTRACE__)}\
+
+              The exception happened while checking this code:
 
               #{Macro.to_string(def_expr)}
 
-              Please report this bug: https://github.com/elixir-lang/elixir/issues
-
-              #{Exception.format_banner(:error, e, __STACKTRACE__)}\
+              In case it is a bug, please report it at: https://github.com/elixir-lang/elixir/issues
               """)
 
             reraise error, __STACKTRACE__
@@ -135,7 +137,7 @@ defmodule Module.Types do
   # Collect relevant information from context and traces to report error
   def error_to_warning(:unable_apply, {mfa, args, expected, signature, stack}, context) do
     {fun, arity} = context.function
-    location = {context.file, get_line(stack), {context.module, fun, arity}}
+    location = {context.file, get_position(stack), {context.module, fun, arity}}
 
     traces = type_traces(stack, context)
     {[signature | args], traces} = lift_all_types([signature | args], traces, context)
@@ -145,7 +147,7 @@ defmodule Module.Types do
 
   def error_to_warning(:unable_unify, {left, right, stack}, context) do
     {fun, arity} = context.function
-    location = {context.file, get_line(stack), {context.module, fun, arity}}
+    location = {context.file, get_position(stack), {context.module, fun, arity}}
 
     traces = type_traces(stack, context)
     {[left, right], traces} = lift_all_types([left, right], traces, context)
@@ -153,7 +155,9 @@ defmodule Module.Types do
     {Module.Types, error, location}
   end
 
-  defp get_line(stack), do: stack.last_expr |> get_meta() |> Keyword.get(:line, 0)
+  defp get_position(stack) do
+    get_meta(stack.last_expr)
+  end
 
   # Collect relevant traces from context.traces using stack.unify_stack
   defp type_traces(stack, context) do
@@ -347,8 +351,8 @@ defmodule Module.Types do
     end)
   end
 
-  defp format_location({file, line, _mfa}) do
-    format_location({file, line})
+  defp format_location({file, position, _mfa}) do
+    format_location({file, position[:line]})
   end
 
   defp format_location({file, line}) do
@@ -409,14 +413,14 @@ defmodule Module.Types do
 
   defp format_message_hint(:inferred_dot) do
     """
-    HINT: "var.field" (without parentheses) implies "var" is a map() while \
+    #{hint()} "var.field" (without parentheses) implies "var" is a map() while \
     "var.fun()" (with parentheses) implies "var" is an atom()
     """
   end
 
   defp format_message_hint(:inferred_bitstring_spec) do
     """
-    HINT: all expressions given to binaries are assumed to be of type \
+    #{hint()} all expressions given to binaries are assumed to be of type \
     integer() unless said otherwise. For example, <<expr>> assumes "expr" \
     is an integer. Pass a modifier, such as <<expr::float>> or <<expr::binary>>, \
     to change the default behaviour.
@@ -425,11 +429,13 @@ defmodule Module.Types do
 
   defp format_message_hint({:sized_and_unsize_tuples, {size, var}}) do
     """
-    HINT: use pattern matching or "is_tuple(#{Macro.to_string(var)}) and \
+    #{hint()} use pattern matching or "is_tuple(#{Macro.to_string(var)}) and \
     tuple_size(#{Macro.to_string(var)}) == #{size}" to guard a sized tuple.
     """
   end
 
+  defp hint, do: :elixir_errors.prefix(:hint)
+
   defp format_type_hint(type, types, expr, hints) do
     case format_type_hint(type, types, expr) do
       {message, hint} -> {message, [hint | hints]}
