@@ -3,36 +3,37 @@ defmodule Task.Supervised do
   @ref_timeout 5000
 
   def start(owner, callers, fun) do
-    {:ok, :proc_lib.spawn(__MODULE__, :noreply, [owner, callers, fun])}
+    {:ok, spawn(__MODULE__, :noreply, [owner, get_ancestors(), callers, fun])}
   end
 
   def start_link(owner, callers, fun) do
-    {:ok, :proc_lib.spawn_link(__MODULE__, :noreply, [owner, callers, fun])}
+    {:ok, spawn_link(__MODULE__, :noreply, [owner, get_ancestors(), callers, fun])}
   end
 
-  def start_link(owner, callers, monitor, fun) do
-    {:ok, :proc_lib.spawn_link(__MODULE__, :reply, [owner, callers, monitor, fun])}
+  def start_link(owner, monitor) do
+    {:ok, spawn_link(__MODULE__, :reply, [owner, get_ancestors(), monitor])}
   end
 
-  def reply({_, _, owner_pid} = owner, callers, monitor, mfa) do
-    initial_call(mfa)
-    put_callers(callers)
+  def reply({_, _, owner_pid} = owner, ancestors, monitor) do
+    put_ancestors(ancestors)
 
     case monitor do
       :monitor ->
         mref = Process.monitor(owner_pid)
-        reply(owner, owner_pid, mref, @ref_timeout, mfa)
+        reply(owner, owner_pid, mref, @ref_timeout)
 
       :nomonitor ->
-        reply(owner, owner_pid, nil, :infinity, mfa)
+        reply(owner, owner_pid, nil, :infinity)
     end
   end
 
-  defp reply(owner, owner_pid, mref, timeout, mfa) do
+  defp reply(owner, owner_pid, mref, timeout) do
     receive do
-      {^owner_pid, ref} ->
-        _ = if mref, do: Process.demonitor(mref, [:flush])
-        send(owner_pid, {ref, invoke_mfa(owner, mfa)})
+      {^owner_pid, ref, reply_to, callers, mfa} ->
+        put_initial_call(mfa)
+        put_callers(callers)
+        _ = is_reference(mref) && Process.demonitor(mref, [:flush])
+        send(reply_to, {ref, invoke_mfa(owner, mfa)})
 
       {:DOWN, ^mref, _, _, reason} ->
         exit({:shutdown, reason})
@@ -61,24 +62,34 @@ defmodule Task.Supervised do
     end
   end
 
-  def noreply(owner, callers, mfa) do
-    initial_call(mfa)
+  def noreply(owner, ancestors, callers, mfa) do
+    put_initial_call(mfa)
+    put_ancestors(ancestors)
     put_callers(callers)
     invoke_mfa(owner, mfa)
   end
 
+  defp get_ancestors() do
+    case :erlang.get(:"$ancestors") do
+      ancestors when is_list(ancestors) -> [self() | ancestors]
+      _ -> [self()]
+    end
+  end
+
+  defp put_ancestors(ancestors) do
+    Process.put(:"$ancestors", ancestors)
+  end
+
   defp put_callers(callers) do
     Process.put(:"$callers", callers)
   end
 
-  defp initial_call(mfa) do
+  defp put_initial_call(mfa) do
     Process.put(:"$initial_call", get_initial_call(mfa))
   end
 
   defp get_initial_call({:erlang, :apply, [fun, []]}) when is_function(fun, 0) do
-    {:module, module} = Function.info(fun, :module)
-    {:name, name} = Function.info(fun, :name)
-    {module, name, 0}
+    :erlang.fun_info_mfa(fun)
   end
 
   defp get_initial_call({mod, fun, args}) do
@@ -117,10 +128,14 @@ defmodule Task.Supervised do
           }
         )
 
-        :erlang.raise(kind, value, __STACKTRACE__)
+        :erlang.raise(:exit, exit_reason(kind, value, __STACKTRACE__), __STACKTRACE__)
     end
   end
 
+  defp exit_reason(:error, reason, stacktrace), do: {reason, stacktrace}
+  defp exit_reason(:exit, reason, _stacktrace), do: reason
+  defp exit_reason(:throw, reason, stacktrace), do: {{:nocatch, reason}, stacktrace}
+
   defp log_value(:throw, value), do: {:nocatch, value}
   defp log_value(_, value), do: value
 
@@ -136,10 +151,10 @@ defmodule Task.Supervised do
         }
       }) do
     message =
-      '** Task ~p terminating~n' ++
-        '** Started from ~p~n' ++
-        '** When function  == ~p~n' ++
-        '**      arguments == ~p~n' ++ '** Reason for termination == ~n' ++ '** ~p~n'
+      ~c"** Task ~p terminating~n" ++
+        ~c"** Started from ~p~n" ++
+        ~c"** When function  == ~p~n" ++
+        ~c"**      arguments == ~p~n" ++ ~c"** Reason for termination == ~n" ++ ~c"** ~p~n"
 
     {message, [starter, name, fun, args, get_reason(reason)]}
   end
@@ -154,7 +169,7 @@ defmodule Task.Supervised do
   defp get_reason({:undef, [{mod, fun, args, _info} | _] = stacktrace} = reason)
        when is_atom(mod) and is_atom(fun) do
     cond do
-      :code.is_loaded(mod) === false ->
+      not Code.loaded?(mod) ->
         {:"module could not be loaded", stacktrace}
 
       is_list(args) and not function_exported?(mod, fun, length(args)) ->
@@ -174,14 +189,19 @@ defmodule Task.Supervised do
 
   ## Stream
 
-  def stream(enumerable, acc, reducer, mfa, options, spawn) do
+  def stream(enumerable, acc, reducer, callers, mfa, options, spawn) do
     next = &Enumerable.reduce(enumerable, &1, fn x, acc -> {:suspend, [x | acc]} end)
     max_concurrency = Keyword.get(options, :max_concurrency, System.schedulers_online())
+
+    unless is_integer(max_concurrency) and max_concurrency > 0 do
+      raise ArgumentError, ":max_concurrency must be an integer greater than zero"
+    end
+
     ordered? = Keyword.get(options, :ordered, true)
     timeout = Keyword.get(options, :timeout, 5000)
     on_timeout = Keyword.get(options, :on_timeout, :exit)
+    zip_input_on_exit? = Keyword.get(options, :zip_input_on_exit, false)
     parent = self()
-    callers = get_callers()
 
     {:trap_exit, trap_exit?} = Process.info(self(), :trap_exit)
 
@@ -192,7 +212,7 @@ defmodule Task.Supervised do
 
     {monitor_pid, monitor_ref} =
       Process.spawn(
-        fn -> stream_monitor(callers, mfa, spawn, trap_exit?, timeout) end,
+        fn -> stream_monitor(parent, spawn, trap_exit?, timeout) end,
         spawn_opts
       )
 
@@ -207,7 +227,10 @@ defmodule Task.Supervised do
       monitor_ref: monitor_ref,
       ordered: ordered?,
       timeout: timeout,
-      on_timeout: on_timeout
+      on_timeout: on_timeout,
+      zip_input_on_exit: zip_input_on_exit?,
+      callers: callers,
+      mfa: mfa
     }
 
     stream_reduce(
@@ -221,16 +244,8 @@ defmodule Task.Supervised do
     )
   end
 
-  defp get_callers do
-    case :erlang.get(:"$callers") do
-      [_ | _] = list -> [self() | list]
-      _ -> [self()]
-    end
-  end
-
   defp stream_reduce({:halt, acc}, _max, _spawned, _delivered, _waiting, next, config) do
-    %{monitor_pid: monitor_pid, monitor_ref: monitor_ref, timeout: timeout} = config
-    stream_close(monitor_pid, monitor_ref, timeout)
+    stream_close(config)
     is_function(next) && next.({:halt, []})
     {:halted, acc}
   end
@@ -243,13 +258,7 @@ defmodule Task.Supervised do
   # All spawned, all delivered, next is :done.
   defp stream_reduce({:cont, acc}, _max, spawned, delivered, _waiting, next, config)
        when spawned == delivered and next == :done do
-    %{
-      monitor_pid: monitor_pid,
-      monitor_ref: monitor_ref,
-      timeout: timeout
-    } = config
-
-    stream_close(monitor_pid, monitor_ref, timeout)
+    stream_close(config)
     {:done, acc}
   end
 
@@ -263,6 +272,7 @@ defmodule Task.Supervised do
       monitor_ref: monitor_ref,
       timeout: timeout,
       on_timeout: on_timeout,
+      zip_input_on_exit: zip_input_on_exit?,
       ordered: ordered?
     } = config
 
@@ -272,7 +282,7 @@ defmodule Task.Supervised do
       # this response when the replying task dies (we'll see this in the :down
       # message).
       {{^monitor_ref, position}, reply} ->
-        %{^position => {pid, :running}} = waiting
+        %{^position => {pid, :running, _element}} = waiting
         waiting = Map.put(waiting, position, {pid, {:ok, reply}})
         stream_reduce({:cont, acc}, max, spawned, delivered, waiting, next, config)
 
@@ -290,20 +300,20 @@ defmodule Task.Supervised do
               ok
 
             # If the task exited by itself before replying, we emit {:exit, reason}.
-            %{^position => {_, :running}}
+            %{^position => {_, :running, element}}
             when kind == :down ->
-              {:exit, reason}
+              if zip_input_on_exit?, do: {:exit, {element, reason}}, else: {:exit, reason}
 
             # If the task timed out before replying, we either exit (on_timeout: :exit)
             # or emit {:exit, :timeout} (on_timeout: :kill_task) (note the task is already
             # dead at this point).
-            %{^position => {_, :running}}
+            %{^position => {_, :running, element}}
             when kind == :timed_out ->
               if on_timeout == :exit do
                 stream_cleanup_inbox(monitor_pid, monitor_ref)
                 exit({:timeout, {__MODULE__, :stream, [timeout]}})
               else
-                {:exit, :timeout}
+                if zip_input_on_exit?, do: {:exit, {element, :timeout}}, else: {:exit, :timeout}
               end
           end
 
@@ -312,33 +322,32 @@ defmodule Task.Supervised do
           stream_deliver({:cont, acc}, max + 1, spawned, delivered, waiting, next, config)
         else
           pair = deliver_now(result, acc, next, config)
+          waiting = Map.delete(waiting, position)
           stream_reduce(pair, max + 1, spawned, delivered + 1, waiting, next, config)
         end
 
       # The monitor process died. We just cleanup the messages from the monitor
       # process and exit.
-      {:DOWN, ^monitor_ref, _, ^monitor_pid, reason} ->
+      {:DOWN, ^monitor_ref, _, _, reason} ->
         stream_cleanup_inbox(monitor_pid, monitor_ref)
         exit({reason, {__MODULE__, :stream, [timeout]}})
     end
   end
 
   defp stream_reduce({:cont, acc}, max, spawned, delivered, waiting, next, config) do
-    %{monitor_pid: monitor_pid, monitor_ref: monitor_ref, timeout: timeout} = config
-
     try do
       next.({:cont, []})
     catch
       kind, reason ->
-        stream_close(monitor_pid, monitor_ref, timeout)
+        stream_close(config)
         :erlang.raise(kind, reason, __STACKTRACE__)
     else
       {:suspended, [value], next} ->
-        waiting = stream_spawn(value, spawned, waiting, monitor_pid, monitor_ref, timeout)
+        waiting = stream_spawn(value, spawned, waiting, config)
         stream_reduce({:cont, acc}, max - 1, spawned + 1, delivered, waiting, next, config)
 
       {_, [value]} ->
-        waiting = stream_spawn(value, spawned, waiting, monitor_pid, monitor_ref, timeout)
+        waiting = stream_spawn(value, spawned, waiting, config)
         stream_reduce({:cont, acc}, max - 1, spawned + 1, delivered, waiting, :done, config)
 
       {_, []} ->
@@ -347,19 +356,14 @@ defmodule Task.Supervised do
   end
 
   defp deliver_now(reply, acc, next, config) do
-    %{
-      reducer: reducer,
-      monitor_pid: monitor_pid,
-      monitor_ref: monitor_ref,
-      timeout: timeout
-    } = config
+    %{reducer: reducer} = config
 
     try do
       reducer.(reply, acc)
     catch
       kind, reason ->
         is_function(next) && next.({:halt, []})
-        stream_close(monitor_pid, monitor_ref, timeout)
+        stream_close(config)
         :erlang.raise(kind, reason, __STACKTRACE__)
     end
   end
@@ -374,12 +378,7 @@ defmodule Task.Supervised do
   end
 
   defp stream_deliver({:cont, acc}, max, spawned, delivered, waiting, next, config) do
-    %{
-      reducer: reducer,
-      monitor_pid: monitor_pid,
-      monitor_ref: monitor_ref,
-      timeout: timeout
-    } = config
+    %{reducer: reducer} = config
 
     case waiting do
       %{^delivered => {:done, reply}} ->
@@ -388,19 +387,12 @@ defmodule Task.Supervised do
         catch
           kind, reason ->
             is_function(next) && next.({:halt, []})
-            stream_close(monitor_pid, monitor_ref, timeout)
+            stream_close(config)
             :erlang.raise(kind, reason, __STACKTRACE__)
         else
           pair ->
-            stream_deliver(
-              pair,
-              max,
-              spawned,
-              delivered + 1,
-              Map.delete(waiting, delivered),
-              next,
-              config
-            )
+            waiting = Map.delete(waiting, delivered)
+            stream_deliver(pair, max, spawned, delivered + 1, waiting, next, config)
         end
 
       %{} ->
@@ -408,7 +400,7 @@ defmodule Task.Supervised do
     end
   end
 
-  defp stream_close(monitor_pid, monitor_ref, timeout) do
+  defp stream_close(%{monitor_pid: monitor_pid, monitor_ref: monitor_ref, timeout: timeout}) do
     send(monitor_pid, {:stop, monitor_ref})
 
     receive do
@@ -445,16 +437,27 @@ defmodule Task.Supervised do
 
   # This function spawns a task for the given "value", and puts the pid of this
   # new task in the map of "waiting" tasks, which is returned.
-  defp stream_spawn(value, spawned, waiting, monitor_pid, monitor_ref, timeout) do
-    send(monitor_pid, {:spawn, spawned, value})
+  defp stream_spawn(value, spawned, waiting, config) do
+    %{
+      monitor_pid: monitor_pid,
+      monitor_ref: monitor_ref,
+      timeout: timeout,
+      callers: callers,
+      mfa: mfa,
+      zip_input_on_exit: zip_input_on_exit?
+    } = config
+
+    send(monitor_pid, {:spawn, spawned})
 
     receive do
       {:spawned, {^monitor_ref, ^spawned}, pid} ->
-        send(pid, {self(), {monitor_ref, spawned}})
-        Map.put(waiting, spawned, {pid, :running})
+        mfa_with_value = normalize_mfa_with_arg(mfa, value)
+        send(pid, {self(), {monitor_ref, spawned}, self(), callers, mfa_with_value})
+        stored_value = if zip_input_on_exit?, do: value, else: nil
+        Map.put(waiting, spawned, {pid, :running, stored_value})
 
       {:max_children, ^monitor_ref} ->
-        stream_close(monitor_pid, monitor_ref, timeout)
+        stream_close(config)
 
         raise """
         reached the maximum number of tasks for this task supervisor. The maximum number \
@@ -471,7 +474,7 @@ defmodule Task.Supervised do
     end
   end
 
-  defp stream_monitor([parent_pid | _] = callers, mfa, spawn, trap_exit?, timeout) do
+  defp stream_monitor(parent_pid, spawn, trap_exit?, timeout) do
     Process.flag(:trap_exit, trap_exit?)
     parent_ref = Process.monitor(parent_pid)
 
@@ -481,9 +484,8 @@ defmodule Task.Supervised do
     receive do
       {^parent_pid, monitor_ref} ->
         config = %{
-          callers: callers,
+          parent_pid: parent_pid,
           parent_ref: parent_ref,
-          mfa: mfa,
           spawn: spawn,
           monitor_ref: monitor_ref,
           timeout: timeout
@@ -498,9 +500,8 @@ defmodule Task.Supervised do
 
   defp stream_monitor_loop(running_tasks, config) do
     %{
-      callers: [parent_pid | _] = callers,
-      mfa: mfa,
       spawn: spawn,
+      parent_pid: parent_pid,
       monitor_ref: monitor_ref,
       timeout: timeout
     } = config
@@ -508,8 +509,8 @@ defmodule Task.Supervised do
     receive do
       # The parent process is telling us to spawn a new task to process
       # "value". We spawn it and notify the parent about its pid.
-      {:spawn, position, value} ->
-        case spawn.(callers, normalize_mfa_with_arg(mfa, value)) do
+      {:spawn, position} ->
+        case spawn.() do
           {:ok, type, pid} ->
             ref = Process.monitor(pid)
 
