@@ -7,7 +7,7 @@
 -export([
   string_to_tokens/5, tokens_to_quoted/3, 'string_to_quoted!'/5,
   env_for_eval/1, quoted_to_erl/2, eval_forms/3, eval_quoted/3,
-  eval_quoted/4, eval_external_handler/3
+  eval_quoted/4, eval_local_handler/2, eval_external_handler/3
 ]).
 -include("elixir.hrl").
 -define(system, 'Elixir.System').
@@ -90,7 +90,7 @@ start(_Type, _Args) ->
     {ignore_already_consolidated, false},
     {ignore_module_conflict, false},
     {on_undefined_variable, raise},
-    {parser_options, []},
+    {parser_options, [{columns, true}]},
     {debug_info, true},
     {warnings_as_errors, false},
     {relative_paths, true},
@@ -177,6 +177,23 @@ check_file_encoding(Encoding) ->
 
 %% Boot and process given options. Invoked by Elixir's script.
 
+%% TODO: Delete prim_tty branches and -user on Erlang/OTP 26.
+%% TODO: Remove IEx.CLI module
+%% TODO: Replace "-user elixir" and "-s elixir start_$MODE"
+%% by calls to "-s elixir cli" and "-e iex:cli()"
+
+start() ->
+  case code:ensure_loaded(prim_tty) of
+    {module, _} ->
+      user_drv:start(#{initial_shell => noshell});
+    {error, _} ->
+      case init:get_argument(elixir_root) of
+        {ok, [[Root]]} -> code:add_patha(Root ++ "/iex/ebin");
+        _ -> ok
+      end,
+      'Elixir.IEx.CLI':deprecated()
+  end.
+
 start_cli() ->
   {ok, _} = application:ensure_all_started(elixir),
 
@@ -193,27 +210,11 @@ start_cli() ->
   'Elixir.Kernel.CLI':main(init:get_plain_arguments()),
   elixir_config:booted().
 
-%% TODO: Delete prim_tty branches and -user on Erlang/OTP 26
-%% and add "-e :elixir.start_iex" to bin/iex instead of $MODE.
-start() ->
-  case code:ensure_loaded(prim_tty) of
-    {module, _} ->
-      user_drv:start(#{initial_shell => noshell});
-    {error, _} ->
-      case init:get_argument(elixir_root) of
-        {ok, [[Root]]} -> code:add_patha(Root ++ "/iex/ebin");
-        _ -> ok
-      end,
-      'Elixir.IEx.CLI':deprecated()
-  end.
 start_iex() ->
   case code:ensure_loaded(prim_tty) of
     {module, _} ->
       start_cli(),
-      spawn(fun() ->
-        elixir_config:wait_until_booted(),
-        (shell:whereis() =:= undefined) andalso 'Elixir.IEx':cli()
-      end);
+      iex:cli();
 
     {error, _} ->
       ok
@@ -358,6 +359,8 @@ eval_forms(Tree, Binding, OrigE, Opts) ->
           _ -> [Erl]
         end,
 
+      %% We use remote names so eval works across Elixir versions.
+      LocalHandler = {value, fun ?MODULE:eval_local_handler/2},
       ExternalHandler = eval_external_handler(),
 
       {value, Value, NewBinding} =
@@ -372,7 +375,7 @@ eval_forms(Tree, Binding, OrigE, Opts) ->
           %% The downside is that functions that escape the eval context will no
           %% longer have the original environment they came from.
           erlang:put(?elixir_eval_env, NewE),
-          erl_eval:exprs(Exprs, ErlBinding, none, ExternalHandler)
+          erl_eval:exprs(Exprs, ErlBinding, LocalHandler, ExternalHandler)
         after
           erlang:erase(?elixir_eval_env)
         end,
@@ -385,6 +388,12 @@ eval_forms(Tree, Binding, OrigE, Opts) ->
       {Value, DumpedBinding, NewE#{versioned_vars := DumpedVars}}
   end.
 
+eval_local_handler(FunName, Args) ->
+  {current_stacktrace, Stack} = erlang:process_info(self(), current_stacktrace),
+  Opts = [{module, nil}, {function, FunName}, {arity, length(Args)}, {reason, 'undefined local'}],
+  Exception = 'Elixir.UndefinedFunctionError':exception(Opts),
+  erlang:raise(error, Exception, Stack).
+
 %% TODO: Remove conditional once we require Erlang/OTP 25+.
 -if(?OTP_RELEASE >= 25).
 eval_external_handler() ->
@@ -477,14 +486,12 @@ string_to_tokens(String, StartLine, StartColumn, File, Opts) when is_integer(Sta
     {ok, _Line, _Column, [], Tokens} ->
       {ok, Tokens};
     {ok, _Line, _Column, Warnings, Tokens} ->
-      (lists:keyfind(warnings, 1, Opts) /= {warnings, false}) andalso
+      (lists:keyfind(emit_warnings, 1, Opts) /= {emit_warnings, false}) andalso
         [elixir_errors:erl_warn(L, File, M) || {L, M} <- lists:reverse(Warnings)],
       {ok, Tokens};
-    {error, {Line, Column, {ErrorPrefix, ErrorSuffix}, Token}, _Rest, _Warnings, _SoFar} ->
-      Location = [{line, Line}, {column, Column}],
+    {error, {Location, {ErrorPrefix, ErrorSuffix}, Token}, _Rest, _Warnings, _SoFar} ->
       {error, {Location, {to_binary(ErrorPrefix), to_binary(ErrorSuffix)}, to_binary(Token)}};
-    {error, {Line, Column, Error, Token}, _Rest, _Warnings, _SoFar} ->
-      Location = [{line, Line}, {column, Column}],
+    {error, {Location, Error, Token}, _Rest, _Warnings, _SoFar} ->
       {error, {Location, to_binary(Error), to_binary(Token)}}
   end.
 
@@ -537,8 +544,8 @@ to_binary(Atom) when is_atom(Atom) -> atom_to_binary(Atom).
 
 handle_parsing_opts(File, Opts) ->
   WarningFile =
-    case lists:keyfind(warnings, 1, Opts) of
-      {warnings, false} -> nil;
+    case lists:keyfind(emit_warnings, 1, Opts) of
+      {emit_warnings, false} -> nil;
       _ -> File
     end,
   LiteralEncoder =
