@@ -1,8 +1,8 @@
 -module(elixir_env).
 -include("elixir.hrl").
 -export([
-  new/0, to_caller/1, with_vars/2, reset_vars/1, env_to_ex/1,
-  reset_unused_vars/1, check_unused_vars/2, merge_and_check_unused_vars/3,
+  new/0, to_caller/1, with_vars/2, reset_vars/1, env_to_ex/1, set_prematch_from_config/1,
+  reset_unused_vars/1, check_unused_vars/2, merge_and_check_unused_vars/3, calculate_span/2,
   trace/2, format_error/1,
   reset_read/2, prepare_write/1, close_write/2
 ]).
@@ -45,13 +45,16 @@ with_vars(Env, #{} = Vars) ->
 reset_vars(Env) ->
   Env#{versioned_vars := #{}}.
 
+set_prematch_from_config(#elixir_ex{} = S) ->
+  S#elixir_ex{prematch=elixir_config:get(on_undefined_variable)}.
+
 %% CONVERSIONS
 
 env_to_ex(#{context := match, versioned_vars := Vars}) ->
   Counter = map_size(Vars),
-  #elixir_ex{prematch={Vars, Counter}, vars={Vars, false}, unused={#{}, Counter}};
+  #elixir_ex{prematch={Vars, Counter, none}, vars={Vars, false}, unused={#{}, Counter}};
 env_to_ex(#{versioned_vars := Vars}) ->
-  #elixir_ex{vars={Vars, false}, unused={#{}, map_size(Vars)}}.
+  set_prematch_from_config(#elixir_ex{vars={Vars, false}, unused={#{}, map_size(Vars)}}).
 
 %% VAR HANDLING
 
@@ -82,10 +85,19 @@ reset_unused_vars(#elixir_ex{unused={_Unused, Version}} = S) ->
   S#elixir_ex{unused={#{}, Version}}.
 
 check_unused_vars(#elixir_ex{unused={Unused, _Version}}, E) ->
-  [elixir_errors:form_warn([{line, Line}], E, ?MODULE, {unused_var, Name, Overridden}) ||
-    {{{Name, nil}, _}, {Line, Overridden}} <- maps:to_list(Unused), is_unused_var(Name)],
+  [elixir_errors:file_warn(calculate_span(Meta, Name), E, ?MODULE, {unused_var, Name, Overridden}) ||
+    {{{Name, nil}, _}, {Meta, Overridden}} <- maps:to_list(Unused), is_unused_var(Name)],
   E.
 
+calculate_span(Meta, Name) ->
+  case lists:keyfind(column, 1, Meta) of
+    {column, Column} ->
+      [{span, {?line(Meta), Column + string:length(atom_to_binary(Name))}} | Meta];
+
+    _ ->
+      Meta
+  end.
+
 merge_and_check_unused_vars(S, #elixir_ex{vars={Read, Write}, unused={Unused, _Version}}, E) ->
   #elixir_ex{unused={ClauseUnused, Version}} = S,
   NewUnused = merge_and_check_unused_vars(Read, Unused, ClauseUnused, E),
@@ -103,11 +115,11 @@ merge_and_check_unused_vars(Current, Unused, ClauseUnused, E) ->
           Acc
       end;
 
-    ({{Name, Kind}, _Count}, {Line, Overridden}, Acc) ->
+    ({{Name, Kind}, _Count}, {Meta, Overridden}, Acc) ->
       case (Kind == nil) andalso is_unused_var(Name) of
         true ->
           Warn = {unused_var, Name, Overridden},
-          elixir_errors:form_warn([{line, Line}], E, ?MODULE, Warn);
+          elixir_errors:file_warn(Meta, E, ?MODULE, Warn);
 
         false ->
           ok
@@ -130,6 +142,8 @@ format_error({unused_var, Name, Overridden}) ->
   case atom_to_list(Name) of
     "_" ++ _ ->
       io_lib:format("unknown compiler variable \"~ts\" (expected one of __MODULE__, __ENV__, __DIR__, __CALLER__, __STACKTRACE__)", [Name]);
+    "&" ++ _ ->
+      io_lib:format("variable \"~ts\" is unused (this might happen when using a capture argument as a pattern)", [Name]);
     _ when Overridden ->
       io_lib:format("variable \"~ts\" is unused (there is a variable with the same name in the context, use the pin operator (^) to match on it or prefix this variable with underscore if it is not meant to be used)", [Name]);
     _ ->
