@@ -68,10 +68,7 @@ expand({alias, Meta, [Ref, Opts]}, S, E) ->
   assert_no_match_or_guard_scope(Meta, "alias", S, E),
   {ERef, SR, ER} = expand_without_aliases_report(Ref, S, E),
   {EOpts, ST, ET} = expand_opts(Meta, alias, [as, warn], no_alias_opts(Opts), SR, ER),
-  case lists:member(ERef, ['Elixir.True', 'Elixir.False', 'Elixir.Nil']) of
-    true ->  elixir_errors:file_warn(Meta, E, ?MODULE, {commonly_mistaken_alias, Ref});
-    false -> ok
-  end,
+
   if
     is_atom(ERef) ->
       {ERef, ST, expand_alias(Meta, true, ERef, EOpts, ET)};
@@ -87,11 +84,19 @@ expand({require, Meta, [Ref, Opts]}, S, E) ->
   {ERef, SR, ER} = expand_without_aliases_report(Ref, S, E),
   {EOpts, ST, ET}  = expand_opts(Meta, require, [as, warn], no_alias_opts(Opts), SR, ER),
 
-  if
-    is_atom(ERef) ->
+  %% Add the alias to context_modules if defined is set.
+  %% This is used by defmodule in order to store the defined
+  %% module in context modules.
+  case lists:keyfind(defined, 1, Meta) of
+    {defined, Mod} when is_atom(Mod) ->
+      EA = ET#{context_modules := [Mod | ?key(ET, context_modules)]},
+      {ERef, ST, expand_alias(Meta, false, ERef, EOpts, EA)};
+
+    false when is_atom(ERef) ->
       elixir_aliases:ensure_loaded(Meta, ERef, ET),
       {ERef, ST, expand_require(Meta, ERef, EOpts, ET)};
-    true ->
+
+    false ->
       file_error(Meta, E, ?MODULE, {expected_compile_time_module, require, Ref})
   end;
 
@@ -209,12 +214,12 @@ expand({'&', Meta, [{super, SuperMeta, Args} = Expr]}, S, E) when is_list(Args)
       expand_fn_capture(Meta, Expr, S, E)
   end;
 
-expand({'&', Meta, [{'/', _, [{super, _, Context}, Arity]} = Expr]}, S, E) when is_atom(Context), is_integer(Arity) ->
+expand({'&', Meta, [{'/', ArityMeta, [{super, SuperMeta, Context}, Arity]} = Expr]}, S, E) when is_atom(Context), is_integer(Arity) ->
   assert_no_match_or_guard_scope(Meta, "&", S, E),
 
   case resolve_super(Meta, Arity, E) of
     {Kind, Name, _} when Kind == def; Kind == defp ->
-      {{'&', Meta, [{'/', [], [{Name, [], Context}, Arity]}]}, S, E};
+      {{'&', Meta, [{'/', ArityMeta, [{Name, SuperMeta, Context}, Arity]}]}, S, E};
     _ ->
       expand_fn_capture(Meta, Expr, S, E)
   end;
@@ -380,8 +385,9 @@ expand({Name, Meta, Kind}, S, E) when is_atom(Name), is_atom(Kind) ->
           {{Name, Meta, Kind}, S, E};
 
         _ ->
-          function_error(Meta, E, ?MODULE, {undefined_var, Name, Kind}),
-          {{Name, Meta, Kind}, S, E}
+          SpanMeta =  elixir_env:calculate_span(Meta, Name),
+          function_error(SpanMeta, E, ?MODULE, {undefined_var, Name, Kind}),
+          {{Name, SpanMeta, Kind}, S, E}
       end
   end;
 
@@ -451,6 +457,10 @@ expand(Pid, S, E) when is_pid(Pid) ->
       {Pid, E}
   end;
 
+expand(Zero, S, #{context := match} = E) when is_float(Zero), Zero == 0.0 ->
+  elixir_errors:file_warn([], E, ?MODULE, invalid_match_on_zero_float),
+  {Zero, S, E};
+
 expand(Other, S, E) when is_number(Other); is_atom(Other); is_binary(Other) ->
   {Other, S, E};
 
@@ -501,15 +511,15 @@ resolve_super(Meta, Arity, E) ->
 
 expand_fn_capture(Meta, Arg, S, E) ->
   case elixir_fn:capture(Meta, Arg, S, E) of
-    {{remote, Remote, Fun, Arity}, RemoteMeta, SE, EE} ->
+    {{remote, Remote, Fun, Arity}, RequireMeta, DotMeta, SE, EE} ->
       is_atom(Remote) andalso
-        elixir_env:trace({remote_function, RemoteMeta, Remote, Fun, Arity}, E),
-      AttachedMeta = attach_context_module(Remote, Meta, E),
-      {{'&', AttachedMeta, [{'/', [], [{{'.', [], [Remote, Fun]}, [], []}, Arity]}]}, SE, EE};
-    {{local, Fun, Arity}, _LocalMeta, _SE, #{function := nil}} ->
+        elixir_env:trace({remote_function, RequireMeta, Remote, Fun, Arity}, E),
+      AttachedMeta = attach_context_module(Remote, RequireMeta, E),
+      {{'&', Meta, [{'/', [], [{{'.', DotMeta, [Remote, Fun]}, AttachedMeta, []}, Arity]}]}, SE, EE};
+    {{local, Fun, Arity}, _, _, _SE, #{function := nil}} ->
       file_error(Meta, E, ?MODULE, {undefined_local_capture, Fun, Arity});
-    {{local, Fun, Arity}, _LocalMeta, SE, EE} ->
-      {{'&', Meta, [{'/', [], [{Fun, [], nil}, Arity]}]}, SE, EE};
+    {{local, Fun, Arity}, LocalMeta, _, SE, EE} ->
+      {{'&', Meta, [{'/', [], [{Fun, LocalMeta, nil}, Arity]}]}, SE, EE};
     {expand, Expr, SE, EE} ->
       expand(Expr, SE, EE)
   end.
@@ -844,21 +854,22 @@ expand_local(Meta, Name, Args, _S, #{function := nil} = E) ->
 
 %% Remote
 
-expand_remote(Receiver, DotMeta, Right, Meta, Args, S, SL, #{context := Context} = E) when is_atom(Receiver) or is_tuple(Receiver) ->
+expand_remote(Receiver, DotMeta, Right, Meta, Args, S, SL, #{context := Context} = E)
+    when is_atom(Receiver) or is_tuple(Receiver) ->
   assert_no_clauses(Right, Meta, Args, E),
 
-  case {Context, lists:keyfind(no_parens, 1, Meta)} of
-    {guard, NoParens} when is_tuple(Receiver) ->
-      (NoParens /= {no_parens, true}) andalso
+  if
+    Context =:= guard, is_tuple(Receiver) ->
+      (lists:keyfind(no_parens, 1, Meta) /= {no_parens, true}) andalso
         function_error(Meta, E, ?MODULE, {parens_map_lookup, Receiver, Right, guard_context(S)}),
 
       {{{'.', DotMeta, [Receiver, Right]}, Meta, []}, SL, E};
 
-    _ ->
-      AttachedDotMeta = attach_context_module(Receiver, DotMeta, E),
+    true ->
+      AttachedMeta = attach_context_module(Receiver, Meta, E),
       {EArgs, {SA, _}, EA} = mapfold(fun expand_arg/3, {SL, S}, E, Args),
 
-      case rewrite(Context, Receiver, AttachedDotMeta, Right, Meta, EArgs, S) of
+      case rewrite(Context, Receiver, DotMeta, Right, AttachedMeta, EArgs, S) of
         {ok, Rewritten} ->
           maybe_warn_comparison(Rewritten, Args, E),
           {Rewritten, elixir_env:close_write(SA, S), EA};
@@ -885,6 +896,9 @@ attach_context_module(Receiver, Meta, #{context_modules := ContextModules}) ->
     false -> Meta
   end.
 
+% Signed numbers can be rewritten no matter the context
+rewrite(_, erlang, _, '+', _, [Arg], _S) when is_number(Arg) -> {ok, Arg};
+rewrite(_, erlang, _, '-', _, [Arg], _S) when is_number(Arg) -> {ok, -Arg};
 rewrite(match, Receiver, DotMeta, Right, Meta, EArgs, _S) ->
   elixir_rewrite:match_rewrite(Receiver, DotMeta, Right, Meta, EArgs);
 rewrite(guard, Receiver, DotMeta, Right, Meta, EArgs, S) ->
@@ -973,29 +987,22 @@ expand_require(Meta, Ref, Opts, E) ->
   RE = E#{requires := ordsets:add_element(Ref, ?key(E, requires))},
   expand_alias(Meta, false, Ref, Opts, RE).
 
-expand_alias(Meta, IncludeByDefault, Ref, Opts, #{context_modules := Context} = E) ->
-  New = expand_as(lists:keyfind(as, 1, Opts), Meta, IncludeByDefault, Ref, E),
+expand_alias(Meta, IncludeByDefault, Ref, Opts, E) ->
+  case expand_as(lists:keyfind(as, 1, Opts), Meta, IncludeByDefault, Ref, E) of
+    {ok, New} ->
+      {Aliases, MacroAliases} = elixir_aliases:store(Meta, New, Ref, Opts, E),
+      E#{aliases := Aliases, macro_aliases := MacroAliases};
 
-  %% Add the alias to context_modules if defined is set.
-  %% This is used by defmodule in order to store the defined
-  %% module in context modules.
-  NewContext =
-    case lists:keyfind(defined, 1, Meta) of
-      {defined, Mod} when is_atom(Mod) -> [Mod | Context];
-      false -> Context
-    end,
+    error ->
+      E
+  end.
 
-  {Aliases, MacroAliases} = elixir_aliases:store(Meta, New, Ref, Opts, E),
-  E#{aliases := Aliases, macro_aliases := MacroAliases, context_modules := NewContext}.
-
-expand_as({as, nil}, _Meta, _IncludeByDefault, Ref, _E) ->
-  Ref;
 expand_as({as, Atom}, Meta, _IncludeByDefault, _Ref, E) when is_atom(Atom), not is_boolean(Atom) ->
   case atom_to_list(Atom) of
     "Elixir." ++ ([FirstLetter | _] = Rest) when FirstLetter >= $A, FirstLetter =< $Z ->
       case string:tokens(Rest, ".") of
         [_] ->
-          Atom;
+          {ok, Atom};
         _ ->
           file_error(Meta, E, ?MODULE, {invalid_alias_for_as, nested_alias, Atom})
       end;
@@ -1006,10 +1013,10 @@ expand_as(false, Meta, IncludeByDefault, Ref, E) ->
   if
     IncludeByDefault ->
       case elixir_aliases:last(Ref) of
-        {ok, NewRef} -> NewRef;
+        {ok, NewRef} -> {ok, NewRef};
         error -> file_error(Meta, E, ?MODULE, {invalid_alias_module, Ref})
       end;
-    true -> Ref
+    true -> error
   end;
 expand_as({as, Other}, Meta, _IncludeByDefault, _Ref, E) ->
   file_error(Meta, E, ?MODULE, {invalid_alias_for_as, not_alias, Other}).
@@ -1024,6 +1031,12 @@ expand_without_aliases_report(Other, S, E) ->
 expand_aliases({'__aliases__', Meta, _} = Alias, S, E, Report) ->
   case elixir_aliases:expand_or_concat(Alias, E) of
     Receiver when is_atom(Receiver) ->
+      if
+        Receiver =:= 'Elixir.True'; Receiver =:= 'Elixir.False'; Receiver =:= 'Elixir.Nil' ->
+          elixir_errors:file_warn(Meta, E, ?MODULE, {commonly_mistaken_alias, Receiver});
+        true ->
+          ok
+      end,
       Report andalso elixir_env:trace({alias_reference, Meta, Receiver}, E),
       {Receiver, S, E};
 
@@ -1155,6 +1168,8 @@ assert_no_underscore_clause_in_cond(_Other, _E) ->
 guard_context(#elixir_ex{prematch={_, _, {bitsize, _}}}) -> "bitstring size specifier";
 guard_context(_) -> "guards".
 
+format_error(invalid_match_on_zero_float) ->
+  "pattern matching on 0.0 is equivalent to matching only on +0.0 from Erlang/OTP 27+. Instead you must match on +0.0 or -0.0";
 format_error({useless_literal, Term}) ->
   io_lib:format("code block contains unused literal ~ts "
                 "(remove the literal or assign it to _ to avoid warnings)",
