@@ -5,6 +5,8 @@ defmodule Module.Types.TypesTest do
   alias Module.Types
   alias Module.Types.{Pattern, Expr}
 
+  @hint :elixir_errors.prefix(:hint)
+
   defmacro warning(patterns \\ [], guards \\ [], body) do
     min_line = min_line(patterns ++ guards ++ [body])
     patterns = reset_line(patterns, min_line)
@@ -128,17 +130,17 @@ defmodule Module.Types.TypesTest do
              in expression:
 
                  # types_test.ex:1
-                 <<..., var::binary()>>
+                 <<..., var::binary>>
 
              where "var" was given the type integer() in:
 
                  # types_test.ex:1
-                 <<var::integer(), ...>>
+                 <<var::integer, ...>>
 
              where "var" was given the type binary() in:
 
                  # types_test.ex:1
-                 <<..., var::binary()>>
+                 <<..., var::binary>>
              """
     end
 
@@ -187,36 +189,6 @@ defmodule Module.Types.TypesTest do
              """
     end
 
-    test "warns on guards with multiple variables" do
-      string = warning([x = y], [is_integer(x) and is_binary(y)], {x, y})
-
-      assert string == """
-             incompatible types:
-
-                 integer() !~ binary()
-
-             in expression:
-
-                 # types_test.ex:1
-                 is_binary(y)
-
-             where "y" was given the same type as "x" in:
-
-                 # types_test.ex:1
-                 x = y
-
-             where "y" was given the type binary() in:
-
-                 # types_test.ex:1
-                 is_binary(y)
-
-             where "x" was given the type integer() in:
-
-                 # types_test.ex:1
-                 is_integer(x)
-             """
-    end
-
     test "warns on guards from cases unless generated" do
       string =
         warning(
@@ -243,36 +215,6 @@ defmodule Module.Types.TypesTest do
       assert string == :none
     end
 
-    test "only show relevant traces in warning" do
-      string = warning([x = y, z], [is_integer(x) and is_binary(y) and is_boolean(z)], {x, y, z})
-
-      assert string == """
-             incompatible types:
-
-                 integer() !~ binary()
-
-             in expression:
-
-                 # types_test.ex:1
-                 is_binary(y)
-
-             where "y" was given the same type as "x" in:
-
-                 # types_test.ex:1
-                 x = y
-
-             where "y" was given the type binary() in:
-
-                 # types_test.ex:1
-                 is_binary(y)
-
-             where "x" was given the type integer() in:
-
-                 # types_test.ex:1
-                 is_integer(x)
-             """
-    end
-
     test "check body" do
       string = warning([x], [is_integer(x)], :foo = x)
 
@@ -321,7 +263,7 @@ defmodule Module.Types.TypesTest do
                  # types_test.ex:1
                  <<foo>>
 
-             HINT: all expressions given to binaries are assumed to be of type \
+             #{@hint} all expressions given to binaries are assumed to be of type \
              integer() unless said otherwise. For example, <<expr>> assumes "expr" \
              is an integer. Pass a modifier, such as <<expr::float>> or <<expr::binary>>, \
              to change the default behaviour.
@@ -337,7 +279,7 @@ defmodule Module.Types.TypesTest do
              in expression:
 
                  # types_test.ex:1
-                 <<foo::integer()>>
+                 <<foo::integer>>
 
              where "foo" was given the type binary() in:
 
@@ -347,7 +289,7 @@ defmodule Module.Types.TypesTest do
              where "foo" was given the type integer() in:
 
                  # types_test.ex:1
-                 <<foo::integer()>>
+                 <<foo::integer>>
              """
     end
 
@@ -374,7 +316,7 @@ defmodule Module.Types.TypesTest do
                  # types_test.ex:1
                  {_} = foo
 
-             HINT: use pattern matching or "is_tuple(foo) and tuple_size(foo) == 1" to guard a sized tuple.
+             #{@hint} use pattern matching or "is_tuple(foo) and tuple_size(foo) == 1" to guard a sized tuple.
              """
     end
 
@@ -417,6 +359,25 @@ defmodule Module.Types.TypesTest do
                  foo - :bar
              """
     end
+
+    test "rewrite call" do
+      string = warning([foo], [is_map_key(1, foo)], foo)
+
+      assert string == """
+             expected Kernel.is_map_key/2 to have signature:
+
+                 integer(), var1 -> dynamic()
+
+             but it has signature:
+
+                 %{optional(dynamic()) => dynamic()}, dynamic() -> dynamic()
+
+             in expression:
+
+                 # types_test.ex:1
+                 is_map_key(1, foo)
+             """
+    end
   end
 
   describe "map warnings" do
@@ -443,7 +404,17 @@ defmodule Module.Types.TypesTest do
                  # types_test.ex:5
                  %{"id" => user_id} = user
 
-             where "user" was given the same type as "amount" in:
+             where "amount" was given the type binary() in:
+
+                 # types_test.ex:3
+                 %{"amount" => amount} = event
+
+             where "amount" was given the same type as "user" in:
+
+                 # types_test.ex:4
+                 %{"user" => user} = event
+
+             where "user" was given the type binary() in:
 
                  # types_test.ex:4
                  %{"user" => user} = event
@@ -452,11 +423,6 @@ defmodule Module.Types.TypesTest do
 
                  # types_test.ex:5
                  %{"id" => user_id} = user
-
-             where "amount" was given the type binary() in:
-
-                 # types_test.ex:3
-                 %{"amount" => amount} = event
              """
     end
 
@@ -490,7 +456,7 @@ defmodule Module.Types.TypesTest do
                  # types_test.ex:4
                  :atom = foo
 
-             HINT: "var.field" (without parentheses) implies "var" is a map() while \
+             #{@hint} "var.field" (without parentheses) implies "var" is a map() while \
              "var.fun()" (with parentheses) implies "var" is an atom()
              """
     end
@@ -525,7 +491,7 @@ defmodule Module.Types.TypesTest do
                  # types_test.ex:4
                  module.__struct__
 
-             HINT: "var.field" (without parentheses) implies "var" is a map() while \
+             #{@hint} "var.field" (without parentheses) implies "var" is a map() while \
              "var.fun()" (with parentheses) implies "var" is an atom()
              """
     end
@@ -553,7 +519,7 @@ defmodule Module.Types.TypesTest do
                  # types_test.ex:1
                  foo.__struct__()
 
-             HINT: "var.field" (without parentheses) implies "var" is a map() while \
+             #{@hint} "var.field" (without parentheses) implies "var" is a map() while \
              "var.fun()" (with parentheses) implies "var" is an atom()
              """
     end
@@ -668,7 +634,7 @@ defmodule Module.Types.TypesTest do
                  # types_test.ex:5
                  map2.subkey
 
-             HINT: "var.field" (without parentheses) implies "var" is a map() while "var.fun()" (with parentheses) implies "var" is an atom()
+             #{@hint} "var.field" (without parentheses) implies "var" is a map() while "var.fun()" (with parentheses) implies "var" is an atom()
              """
     end
   end
@@ -700,5 +666,56 @@ defmodule Module.Types.TypesTest do
                )
              ) == :none
     end
+
+    test "map patterns with pinned keys and field access" do
+      assert warning(
+               [x, y],
+               (
+                 key_var = y
+                 %{^key_var => _value} = x
+                 key_var2 = y
+                 %{^key_var2 => _value2} = x
+                 y.z
+               )
+             ) == :none
+    end
+
+    test "map patterns with pinned keys" do
+      assert warning(
+               [x, y],
+               (
+                 key_var = y
+                 %{^key_var => _value} = x
+                 key_var2 = y
+                 %{^key_var2 => _value2} = x
+                 key_var3 = y
+                 %{^key_var3 => _value3} = x
+               )
+             ) == :none
+    end
+
+    test "map updates with var key" do
+      assert warning(
+               [state0, key0],
+               (
+                 state1 = %{state0 | key0 => true}
+                 key1 = key0
+                 state2 = %{state1 | key1 => true}
+                 state2
+               )
+             ) == :none
+    end
+
+    test "nested map updates" do
+      assert warning(
+               [state],
+               (
+                 _foo = state.key.user_id
+                 _bar = state.key.user_id
+                 state = %{state | key: %{state.key | other_id: 1}}
+                 _baz = state.key.user_id
+               )
+             ) == :none
+    end
   end
 end
