@@ -18,8 +18,10 @@ defmodule ModuleTest.ToBeUsed do
 
   defmacro __after_compile__(%Macro.Env{module: ModuleTest.ToUse} = env, bin)
            when is_binary(bin) do
+    # Ensure module is not longer tracked as being loaded
+    false = __MODULE__ in :elixir_module.compiler_modules()
     [] = Macro.Env.vars(env)
-    # IO.puts "HELLO"
+    :ok
   end
 
   defmacro callback(env) do
@@ -33,7 +35,7 @@ end
 
 defmodule ModuleTest.ToUse do
   # Moving the next line around can make tests fail
-  36 = __ENV__.line
+  38 = __ENV__.line
   var = 1
   # Not available in callbacks
   _ = var
@@ -147,7 +149,7 @@ defmodule ModuleTest do
   end
 
   test "retrieves line from use callsite" do
-    assert ModuleTest.ToUse.line() == 41
+    assert ModuleTest.ToUse.line() == 43
   end
 
   ## Callbacks
@@ -332,6 +334,16 @@ defmodule ModuleTest do
     assert Elixir.ModuleTest.NonAtomAlias.hello() == :world
   end
 
+  test "does not leak alias from Elixir root alias" do
+    defmodule Elixir.ModuleTest.ElixirRootAlias do
+      def hello, do: :world
+    end
+
+    refute __ENV__.aliases[Elixir.ModuleTest]
+    refute __ENV__.aliases[Elixir.ElixirRootAlias]
+    assert Elixir.ModuleTest.ElixirRootAlias.hello() == :world
+  end
+
   test "does not warn on captured underscored vars" do
     _unused = 123
 
@@ -357,8 +369,10 @@ defmodule ModuleTest do
         def world, do: true
       end
 
-    assert_raise CompileError, fn ->
-      {:module, Elixir, _, _} = Module.create(Elixir, contents, __ENV__)
+    assert_raise CompileError, ~r/cannot compile module Elixir/, fn ->
+      Code.with_diagnostics(fn ->
+        Module.create(Elixir, contents, __ENV__)
+      end)
     end
   end
 
@@ -509,12 +523,14 @@ defmodule ModuleTest do
     in_module do
       def foo(a, b), do: a + b
 
-      assert {:v1, :def, _,
+      assert {:v1, :def, def_meta,
               [
-                {_, [{:a, _, nil}, {:b, _, nil}], [],
+                {clause_meta, [{:a, _, nil}, {:b, _, nil}], [],
                  {{:., _, [:erlang, :+]}, _, [{:a, _, nil}, {:b, _, nil}]}}
               ]} = Module.get_definition(__MODULE__, {:foo, 2})
 
+      assert [line: _, column: _] = Keyword.take(def_meta, [:line, :column])
+      assert [line: _, column: _] = Keyword.take(clause_meta, [:line, :column])
       assert {:v1, :def, _, []} = Module.get_definition(__MODULE__, {:foo, 2}, skip_clauses: true)
 
       assert Module.delete_definition(__MODULE__, {:foo, 2})
