@@ -156,7 +156,20 @@ defmodule RecordTest do
       )
 
     assert user == {RecordTest, :name, {RecordTest, :inner_name, :inner_age}}
-    assert Process.info(self(), :messages) == {:messages, [:inner_age, :inner_name, :name]}
+    assert for(_ <- 1..3, do: assert_receive(_)) == [:inner_age, :inner_name, :name]
+
+    user =
+      user(
+        name: send(self(), :name),
+        age:
+          user(
+            age: send(self(), :inner_age),
+            name: send(self(), :inner_name)
+          )
+      )
+
+    assert user == {RecordTest, :name, {RecordTest, :inner_name, :inner_age}}
+    assert for(_ <- 1..3, do: assert_receive(_)) == [:name, :inner_age, :inner_name]
   end
 
   Record.defrecord(
@@ -171,7 +184,7 @@ defmodule RecordTest do
     call: MapSet.new(),
     string: "abc",
     binary: <<1, 2, 3>>,
-    charlist: 'abc'
+    charlist: ~c"abc"
   )
 
   test "records with literal defaults and on-the-fly record" do
@@ -186,7 +199,7 @@ defmodule RecordTest do
              call: MapSet.new(),
              string: "abc",
              binary: <<1, 2, 3>>,
-             charlist: 'abc'
+             charlist: ~c"abc"
            ]
 
     assert defaults(defaults(), :struct) == ~D[2016-01-01]
@@ -199,7 +212,7 @@ defmodule RecordTest do
     assert defaults(defaults(), :call) == MapSet.new()
     assert defaults(defaults(), :string) == "abc"
     assert defaults(defaults(), :binary) == <<1, 2, 3>>
-    assert defaults(defaults(), :charlist) == 'abc'
+    assert defaults(defaults(), :charlist) == ~c"abc"
   end
 
   test "records with literal defaults and record in a variable" do
@@ -216,7 +229,7 @@ defmodule RecordTest do
              call: MapSet.new(),
              string: "abc",
              binary: <<1, 2, 3>>,
-             charlist: 'abc'
+             charlist: ~c"abc"
            ]
 
     assert defaults(defaults, :struct) == ~D[2016-01-01]
@@ -229,7 +242,7 @@ defmodule RecordTest do
     assert defaults(defaults, :call) == MapSet.new()
     assert defaults(defaults, :string) == "abc"
     assert defaults(defaults, :binary) == <<1, 2, 3>>
-    assert defaults(defaults, :charlist) == 'abc'
+    assert defaults(defaults, :charlist) == ~c"abc"
   end
 
   test "records with dynamic arguments" do
@@ -274,6 +287,12 @@ defmodule RecordTest do
     refute macro_exported?(__MODULE__, :file_info, 1)
   end
 
+  test "records reflection" do
+    assert %{fields: [:name, :age], kind: :defrecord, name: :user, tag: RecordTest} in @__records__
+
+    assert %{fields: [:date, :time], kind: :defrecord, name: :timestamp, tag: :timestamp} in @__records__
+  end
+
   test "records with no defaults" do
     record = timestamp()
     assert timestamp(record, :date) == nil
