@@ -71,15 +71,12 @@ defmodule RegexTest do
     assert Regex.re_pattern(Regex.compile!("foo")) == Regex.re_pattern(~r"foo")
 
     assert Regex.source(Regex.compile!("\a\b\d\e\f\n\r\s\t\v")) == "\a\b\d\e\f\n\r\s\t\v"
-    assert Regex.source(~r<\a\b\d\e\f\n\r\s\t\v>) == "\a\\b\\d\\e\f\n\r\\s\t\v"
+    assert Regex.source(~r<\a\b\d\e\f\n\r\s\t\v>) == "\\a\\b\\d\\e\\f\\n\\r\\s\\t\\v"
 
     assert Regex.re_pattern(Regex.compile!("\a\b\d\e\f\n\r\s\t\v")) ==
-             Regex.re_pattern(~r"\a\010\177\033\f\n\r \t\v")
+             Regex.re_pattern(~r"\x07\x08\x7F\x1B\x0C\x0A\x0D\x20\x09\x0B")
 
-    assert Regex.source(Regex.compile!("\a\\b\\d\e\f\n\r\\s\t\v")) == "\a\\b\\d\e\f\n\r\\s\t\v"
-    assert Regex.source(~r<\a\\b\\d\\e\f\n\r\\s\t\v>) == "\a\\\\b\\\\d\\\\e\f\n\r\\\\s\t\v"
-
-    assert Regex.re_pattern(Regex.compile!("\a\\b\\d\e\f\n\r\\s\t\v")) ==
+    assert Regex.re_pattern(Regex.compile!("\\a\\b\\d\e\f\\n\\r\\s\\t\\v")) ==
              Regex.re_pattern(~r"\a\b\d\e\f\n\r\s\t\v")
   end
 
@@ -90,10 +87,6 @@ defmodule RegexTest do
     # Non breaking space matches [[:space:]] with Unicode
     assert <<0xA0::utf8>> =~ ~r/[[:space:]]/u
     assert <<0xA0::utf8>> =~ ~r/\s/u
-
-    # Erlang/OTP 23 raises badarg on invalid UTF-8.
-    # Earlier versions simply would not match.
-    assert catch_error(if <<?<, 255, ?>>> =~ ~r/<.>/u, do: flunk("failed"), else: raise("failed"))
     assert <<?<, 255, ?>>> =~ ~r/<.>/
   end
 
@@ -135,6 +128,7 @@ defmodule RegexTest do
 
   test "opts/1" do
     assert Regex.opts(Regex.compile!("foo", "i")) == "i"
+    assert Regex.opts(Regex.compile!("foo", [:caseless])) == [:caseless]
   end
 
   test "names/1" do
@@ -162,10 +156,6 @@ defmodule RegexTest do
     assert Regex.named_captures(~r/c(.)/, "cat") == %{}
   end
 
-  test "sigil R" do
-    assert Regex.match?(~R/f#{1,3}o/, "f#o")
-  end
-
   test "run/2" do
     assert Regex.run(~r"c(d)", "abcd") == ["cd", "d"]
     assert Regex.run(~r"e", "abcd") == nil
@@ -182,6 +172,13 @@ defmodule RegexTest do
     assert Regex.run(~r"e", "abcd", return: :index) == nil
   end
 
+  test "run/3 with :offset" do
+    assert Regex.run(~r"^foo", "foobar", offset: 0) == ["foo"]
+    assert Regex.run(~r"^foo", "foobar", offset: 2) == nil
+    assert Regex.run(~r"^foo", "foobar", offset: 2, return: :index) == nil
+    assert Regex.run(~r"bar", "foobar", offset: 2, return: :index) == [{3, 3}]
+  end
+
   test "run/3 with regexes compiled in different systems" do
     assert Regex.run(@re_21_3_little, "abcd abce", capture: :all_names) == ["d"]
     assert Regex.run(@re_21_3_big, "abcd abce", capture: :all_names) == ["d"]
@@ -205,6 +202,11 @@ defmodule RegexTest do
     assert Regex.scan(~r/c(?<foo>d|e)/, "abcd abce", capture: :all_names) == [["d"], ["e"]]
   end
 
+  test "scan/2 with :offset" do
+    assert Regex.scan(~r"^foo", "foobar", offset: 0) == [["foo"]]
+    assert Regex.scan(~r"^foo", "foobar", offset: 1) == []
+  end
+
   test "scan/2 with regexes compiled in different systems" do
     assert Regex.scan(@re_21_3_little, "abcd abce", capture: :all_names) == [["d"], ["e"]]
     assert Regex.scan(@re_21_3_big, "abcd abce", capture: :all_names) == [["d"], ["e"]]
@@ -284,17 +286,17 @@ defmodule RegexTest do
   end
 
   test "replace/3,4" do
-    assert Regex.replace(~r(d), "abc", "d") == "abc"
-    assert Regex.replace(~r(b), "abc", "d") == "adc"
-    assert Regex.replace(~r(b), "abc", "[\\0]") == "a[b]c"
+    assert Regex.replace(~r/d/, "abc", "d") == "abc"
+    assert Regex.replace(~r/b/, "abc", "d") == "adc"
+    assert Regex.replace(~r/b/, "abc", "[\\0]") == "a[b]c"
     assert Regex.replace(~r[(b)], "abc", "[\\1]") == "a[b]c"
     assert Regex.replace(~r[(b)], "abc", "[\\2]") == "a[]c"
     assert Regex.replace(~r[(b)], "abc", "[\\3]") == "a[]c"
-    assert Regex.replace(~r(b), "abc", "[\\g{0}]") == "a[b]c"
+    assert Regex.replace(~r/b/, "abc", "[\\g{0}]") == "a[b]c"
     assert Regex.replace(~r[(b)], "abc", "[\\g{1}]") == "a[b]c"
 
-    assert Regex.replace(~r(b), "abcbe", "d") == "adcde"
-    assert Regex.replace(~r(b), "abcbe", "d", global: false) == "adcbe"
+    assert Regex.replace(~r/b/, "abcbe", "d") == "adcde"
+    assert Regex.replace(~r/b/, "abcbe", "d", global: false) == "adcbe"
 
     assert Regex.replace(~r/ /, "first third", "\\second\\") == "first\\second\\third"
     assert Regex.replace(~r/ /, "first third", "\\\\second\\\\") == "first\\second\\third"
