@@ -148,6 +148,16 @@ defmodule ExUnit do
   end
 
   defmodule TimeoutError do
+    @moduledoc """
+    Exception raised when a test times out.
+    """
+
+    @typedoc since: "1.16.0"
+    @type t :: %__MODULE__{
+            timeout: non_neg_integer,
+            type: String.t()
+          }
+
     defexception [:timeout, :type]
 
     @impl true
@@ -173,8 +183,7 @@ defmodule ExUnit do
   def start(_type, []) do
     children = [
       ExUnit.Server,
-      ExUnit.CaptureServer,
-      ExUnit.OnExitHandler
+      ExUnit.CaptureServer
     ]
 
     opts = [strategy: :one_for_one, name: ExUnit.Supervisor]
@@ -232,7 +241,10 @@ defmodule ExUnit do
 
     * `:capture_log` - if ExUnit should default to keeping track of log messages
       and print them on test failure. Can be overridden for individual tests via
-      `@tag capture_log: false`. Defaults to `false`;
+      `@tag capture_log: false`. This can also be configured to a specific level
+       with `capture_log: [level: LEVEL]`, for example:
+       `capture_log: [level: :emergency]` to prevent any output from test failures.
+       Defaults to `false`;
 
     * `:colors` - a keyword list of color options to be used by some formatters:
       * `:enabled` - boolean option to enable colors, defaults to `IO.ANSI.enabled?/0`;
@@ -282,6 +294,13 @@ defmodule ExUnit do
       what tests get run. This is typically used by Mix to filter which tests
       should run;
 
+    * `:rand_algorithm` - algorithm to be used when generating the test seed.
+      Available algorithms can be found in Erlang's
+      [`:rand`](https://www.erlang.org/doc/man/rand.html) documentation (see
+      [`:rand.builting_arg/0`](https://www.erlang.org/doc/man/rand.html#type-builtin_alg)).
+      Available since v1.16.0. Before v1.16.0, the algorithm was hard-coded to
+      `:exs1024`. On Elixir v1.16.0 and after, the default changed to `:exsss`;
+
     * `:refute_receive_timeout` - the timeout to be used on `refute_receive`
       calls in milliseconds, defaults to `100`;
 
