@@ -36,7 +36,7 @@ defmodule IEx do
   IEx expects inputs and outputs to be in UTF-8 encoding. This is the
   default for most Unix terminals but it may not be the case on Windows.
   If you are running on Windows and you see incorrect values printed,
-  you may need change the encoding of your current session by running
+  you may need to change the encoding of your current session by running
   `chcp 65001` before calling `iex` (or before calling `iex.bat` if using
   PowerShell).
 
@@ -242,7 +242,7 @@ defmodule IEx do
   Now, try to access the `hello` variable again:
 
       hello
-      ** (UndefinedFunctionError) undefined function hello/0
+      ** (CompileError) undefined variable "hello"
 
   The command above fails because we have switched shells.
   Since shells are isolated from each other, you can't access the
@@ -858,14 +858,11 @@ defmodule IEx do
 
   ## CLI
 
-  @compile {:no_warn_undefined, {:shell, :start_interactive, 1}}
-
   # This is a callback invoked by Erlang shell utilities
   # when someone presses Ctrl+G and adds `s 'Elixir.IEx'`.
-  # Let's consider exposing this as iex:start() and rename
-  # elixir:start_iex() to iex:cli().
   @doc false
   def start(opts \\ [], mfa \\ {IEx, :dont_display_result, []}) do
+    # TODO: Expose this as iex:start() instead on Erlang/OTP 26+
     # TODO: Keep only this branch, delete optional args and mfa,
     # and delete IEx.Server.run_from_shell/2 on Erlang/OTP 26+
     if Code.ensure_loaded?(:prim_tty) do
@@ -890,43 +887,18 @@ defmodule IEx do
     end
   end
 
-  # Manual tests for changing the CLI boot.
-  #
-  # 1. In some situations, we cannot read inputs as IEx boots:
-  #
-  #      $ iex -e ":io.get_line(:foo)"
-  #
-  # 2. In some situations, connecting to a remote node via --remsh
-  #    is not possible. This can be tested by starting two IEx nodes:
-  #
-  #      $ iex --sname foo
-  #      $ iex --sname bar --remsh foo
-  #
-  # 3. When still using --remsh, we need to guarantee the arguments
-  #    are processed on the local node and not the remote one. For such,
-  #    one can replace the last line above by:
-  #
-  #      $ iex --sname bar --remsh foo -e 'IO.inspect node()'
-  #
-  #    And verify that the local node name is printed.
-  #
-  # 4. Finally, in some other circumstances, printing messages may become
-  #    borked. This can be verified with:
-  #
-  #      $ iex -e ":logger.info('foo~nbar', [])"
-  #
-  # By the time those instructions have been written, all tests above pass.
-  @doc false
-  def cli(opts \\ []) do
-    args = :init.get_plain_arguments()
-    opts = opts ++ [dot_iex_path: find_dot_iex(args), on_eof: :halt]
+  # TODO: Make this public when we require Erlang/OTP 26+
+  @compile {:no_warn_undefined, {:shell, :start_interactive, 1}}
 
+  @doc false
+  def shell(opts) do
+    {remote, opts} = Keyword.pop(opts, :remote)
     ref = make_ref()
-    mfa = {__MODULE__, :__cli__, [self(), ref, opts]}
+    mfa = {__MODULE__, :__shell__, [self(), ref, opts]}
 
     shell =
-      if remote = get_remsh(args) do
-        {:remote, remote, mfa}
+      if remote do
+        {:remote, to_charlist(remote), mfa}
       else
         mfa
       end
@@ -934,31 +906,20 @@ defmodule IEx do
     case :shell.start_interactive(shell) do
       :ok ->
         receive do
-          {^ref, shell} -> shell
+          {^ref, shell} -> {:ok, shell}
         after
-          15_000 ->
-            IO.puts(:stderr, "Could not start IEx CLI due to reason: :boot_timeout")
-            System.halt(1)
+          15_000 -> {:error, :timeout}
         end
 
       {:error, reason} ->
-        IO.puts(:stderr, "Could not start IEx CLI due to reason: #{inspect(reason)}")
-        System.halt(1)
+        {:error, reason}
     end
   end
 
   @doc false
-  def __cli__(parent, ref, opts) do
+  def __shell__(parent, ref, opts) do
     pid = start(opts)
     send(parent, {ref, pid})
     pid
   end
-
-  defp find_dot_iex([~c"--dot-iex", h | _]), do: List.to_string(h)
-  defp find_dot_iex([_ | t]), do: find_dot_iex(t)
-  defp find_dot_iex([]), do: nil
-
-  defp get_remsh([~c"--remsh", h | _]), do: h
-  defp get_remsh([_ | t]), do: get_remsh(t)
-  defp get_remsh([]), do: nil
 end
