@@ -29,13 +29,20 @@ defmodule Logger.Translator do
   and the default messages translated by Logger.
   """
 
+  @doc """
+  Callback for translating a logger message.
+  """
+  @callback translate(Logger.level(), Logger.level(), :format | :report, :logger.report()) ::
+              {:ok, iodata, keyword}
+              | {:ok, iodata}
+              | :skip
+              | :none
+
   @doc """
   Built-in translation function.
   """
   def translate(min_level, level, kind, message)
 
-  ## Erlang/OTP 21 and after
-
   def translate(min_level, _level, :report, {:logger, %{label: label} = report}) do
     case label do
       {:gen_server, :terminate} ->
@@ -109,9 +116,8 @@ defmodule Logger.Translator do
     case message do
       # This is no longer emitted by Erlang/OTP but it may be
       # manually emitted by libraries like connection.
-      # TODO: Remove this translation on Elixir v1.14
-      {'** Generic server ' ++ _, [name, last, state, reason | client]} ->
-        opts = Application.get_env(:logger, :translator_inspect_opts)
+      {~c"** Generic server " ++ _, [name, last, state, reason | client]} ->
+        opts = Application.fetch_env!(:logger, :translator_inspect_opts)
         {formatted, reason} = format_reason(reason)
         metadata = [crash_reason: reason] ++ registered_name(name)
 
@@ -126,7 +132,7 @@ defmodule Logger.Translator do
           {:ok, msg, metadata}
         end
 
-      {'Error in process ' ++ _, [pid, node, {reason, stack}]} ->
+      {~c"Error in process " ++ _, [pid, node, {reason, stack}]} ->
         reason = Exception.normalize(:error, reason, stack)
 
         msg = [
@@ -139,7 +145,7 @@ defmodule Logger.Translator do
 
         {:ok, msg, [crash_reason: exit_reason(:error, reason, stack)]}
 
-      {'Error in process ' ++ _, [pid, {reason, stack}]} ->
+      {~c"Error in process " ++ _, [pid, {reason, stack}]} ->
         reason = Exception.normalize(:error, reason, stack)
         msg = ["Process ", inspect(pid), " raised an exception" | format(:error, reason, stack)]
         {:ok, msg, [crash_reason: exit_reason(:error, reason, stack)]}
@@ -176,6 +182,21 @@ defmodule Logger.Translator do
     {:ok, ["Application ", Atom.to_string(app), " started at " | inspect(node)]}
   end
 
+  def translate(_min_level, :debug, :report, {:logger, [formatter_error: formatter] ++ data}) do
+    # TODO: Remove :catched once we require Erlang/OTP 25+
+    case data[:caught] || data[:catched] do
+      {:throw, {:error, good, bad}, stacktrace} ->
+        message =
+          "bad return value from Logger formatter #{inspect(formatter)}, " <>
+            "got #{inspect(bad)} after #{inspect(good)}"
+
+        {:ok, Exception.format(:error, RuntimeError.exception(message), stacktrace)}
+
+      _ ->
+        :none
+    end
+  end
+
   ## Helpers
 
   def translate(_min_level, _level, _kind, _message) do
@@ -183,7 +204,7 @@ defmodule Logger.Translator do
   end
 
   defp report_gen_server_terminate(min_level, report) do
-    inspect_opts = Application.get_env(:logger, :translator_inspect_opts)
+    inspect_opts = Application.fetch_env!(:logger, :translator_inspect_opts)
 
     %{
       client_info: client,
@@ -209,7 +230,7 @@ defmodule Logger.Translator do
   end
 
   defp report_gen_event_terminate(min_level, report) do
-    inspect_opts = Application.get_env(:logger, :translator_inspect_opts)
+    inspect_opts = Application.fetch_env!(:logger, :translator_inspect_opts)
 
     %{
       handler: handler,
@@ -357,15 +378,22 @@ defmodule Logger.Translator do
     []
   end
 
-  defp child_debug(:debug, restart_type: restart, shutdown: shutdown, child_type: type) do
-    ["\nRestart: ", inspect(restart), "\nShutdown: ", inspect(shutdown)] ++
-      ["\nType: ", inspect(type)]
+  defp child_debug(:debug, opts) do
+    for {key, value} <- opts do
+      child_debug_key(key, value)
+    end
   end
 
   defp child_debug(_min_level, _child) do
     []
   end
 
+  defp child_debug_key(:restart_type, value), do: ["\nRestart: " | inspect(value)]
+  defp child_debug_key(:shutdown, value), do: ["\nShutdown: " | inspect(value)]
+  defp child_debug_key(:child_type, value), do: ["\nType: " | inspect(value)]
+  defp child_debug_key(:significant, value), do: if(value, do: "\nSignificant: true", else: [])
+  defp child_debug_key(_, _), do: []
+
   defp report_crash(min_level, [[{:initial_call, initial_call} | crashed], linked]) do
     mfa = initial_call_to_mfa(initial_call)
     report_crash(min_level, crashed, [{:initial_call, mfa}], linked)
@@ -376,11 +404,9 @@ defmodule Logger.Translator do
   end
 
   defp report_crash(min_level, crashed, extra, linked) do
-    [
-      {:pid, pid},
-      {:registered_name, name},
-      {:error_info, {kind, reason, stack}} | crashed
-    ] = crashed
+    {pid, crashed} = Keyword.pop_first(crashed, :pid)
+    {name, crashed} = Keyword.pop_first(crashed, :registered_name)
+    {{kind, reason, stack}, crashed} = Keyword.pop_first(crashed, :error_info)
 
     dictionary = crashed[:dictionary]
     reason = Exception.normalize(kind, reason, stack)
@@ -431,6 +457,10 @@ defmodule Logger.Translator do
     [prefix, "Ancestors: ", inspect(ancestors) | crash_info(min_level, debug, prefix)]
   end
 
+  defp crash_info(min_level, [{:process_label, _} | info], prefix) do
+    crash_info(min_level, info, prefix)
+  end
+
   defp crash_info(:debug, debug, prefix) do
     for {key, value} <- debug do
       crash_debug(key, value, prefix)
@@ -577,7 +607,7 @@ defmodule Logger.Translator do
   defp registered_name(_name), do: []
 
   defp format_mfa(mod, fun, :undefined),
-    do: [inspect(mod), ?., Code.Identifier.inspect_as_function(fun) | "/?"]
+    do: [inspect(mod), ?., Macro.inspect_atom(:remote_call, fun) | "/?"]
 
   defp format_mfa(mod, fun, args),
     do: Exception.format_mfa(mod, fun, args)
