@@ -352,10 +352,11 @@ defmodule Mix do
     * `MIX_INSTALL_DIR` - (since v1.12.0) specifies directory where `Mix.install/2` keeps
        install cache
 
-    * `MIX_INSTALL_FORCE` - (since v1.13.0) runs `Mix.install/2` with empty install cache
-
     * `MIX_PATH` - appends extra code paths
 
+    * `MIX_PROFILE` - a list of comma-separated Mix tasks to profile the time spent on
+      functions by the process running the task
+
     * `MIX_QUIET` - does not print information messages to the terminal
 
     * `MIX_REBAR3` - path to rebar3 command that overrides the one Mix installs
@@ -376,6 +377,8 @@ defmodule Mix do
   """
 
   @mix_install_project __MODULE__.InstallProject
+  @mix_install_app :mix_install
+  @mix_install_app_string Atom.to_string(@mix_install_app)
 
   use Application
 
@@ -468,7 +471,7 @@ defmodule Mix do
   """
   @spec compilers() :: [atom()]
   def compilers do
-    [:yecc, :leex, :erlang, :elixir, :app]
+    [:erlang, :elixir, :app]
   end
 
   @doc """
@@ -594,34 +597,50 @@ defmodule Mix do
   Ensures the given application from Erlang/OTP or Elixir and its dependencies
   are available in the path.
 
-  This is mostly used for Mix internal needs. In your own tasks, you should
-  list the Erlang application dependencies under the `:extra_applications`
-  section of your `mix.exs`.
+  Generally speaking, you should list the Erlang application dependencies under
+  the `:extra_applications` section of your `mix.exs`. This must only be used by
+  Mix tasks which wish to avoid depending on Erlang/Elixir for certain reasons.
+
+  This function does not start the given applications.
   """
   @doc since: "1.15.0"
   def ensure_application!(app) when is_atom(app) do
-    ensure_application!(app, Mix.State.builtin_apps(), [])
+    ensure_application!(app, Mix.State.builtin_apps(), [], %{})
     :ok
   end
 
-  defp ensure_application!(app, builtin_apps, optional) do
-    case builtin_apps do
-      %{^app => path} ->
-        Code.prepend_path(path, cache: true)
-        Application.load(app)
-        optional = List.wrap(Application.spec(app, :optional_applications))
-
-        Application.spec(app, :applications)
-        |> List.wrap()
-        |> Enum.each(&ensure_application!(&1, builtin_apps, optional))
+  defp ensure_application!(app, builtin_apps, optional, seen) do
+    case seen do
+      %{^app => _} ->
+        seen
 
       %{} ->
-        unless app in optional do
-          Mix.raise(
-            "The application \"#{app}\" could not be found. This may happen if your " <>
-              "Operating System broke Erlang into multiple packages and may be fixed " <>
-              "by installing the missing \"erlang-dev\" and \"erlang-#{app}\" packages"
-          )
+        seen = Map.put(seen, app, true)
+
+        case builtin_apps do
+          %{^app => path} ->
+            Code.prepend_path(path, cache: true)
+            Application.load(app)
+
+            required = List.wrap(Application.spec(app, :applications))
+            optional = List.wrap(Application.spec(app, :optional_applications))
+
+            Enum.reduce(
+              required ++ optional,
+              seen,
+              &ensure_application!(&1, builtin_apps, optional, &2)
+            )
+
+          %{} ->
+            unless app in optional do
+              Mix.raise(
+                "The application \"#{app}\" could not be found. This may happen if your " <>
+                  "Operating System broke Erlang into multiple packages and may be fixed " <>
+                  "by installing the missing \"erlang-dev\" and \"erlang-#{app}\" packages"
+              )
+            end
+
+            seen
         end
     end
   end
@@ -642,9 +661,6 @@ defmodule Mix do
   This function can only be called outside of a Mix project and only with the
   same dependencies in the given VM.
 
-  **Note:** this feature is currently experimental and it may change
-  in future releases.
-
   ## Options
 
     * `:force` - if `true`, runs with empty install cache. This is useful when you want
@@ -669,7 +685,7 @@ defmodule Mix do
       configuration. The configuration is part of the `Mix.install/2` cache, so
       different configurations will lead to different apps. For this reason, you
       want to minimize the amount of configuration set through this option.
-      Use `Application.put_all_env/4` for setting other runtime configuration.
+      Use `Application.put_all_env/2` for setting other runtime configuration.
 
     * `:config_path` (since v1.14.0) - path to a configuration file. If a `runtime.exs`
       file exists in the same directory as the given path, it is loaded too.
@@ -760,6 +776,21 @@ defmodule Mix do
   The contents inside `defmodule` will only be expanded and executed
   after `Mix.install/2` runs, which means that any struct, macros,
   and imports will be correctly handled.
+
+  ## Environment variables
+
+  The `MIX_INSTALL_DIR` environment variable configures the directory that
+  caches all `Mix.install/2`.
+
+  The `MIX_INSTALL_FORCE` is available since Elixir v1.13.0 and forces
+  `Mix.install/2` to discard any previously cached entry of the current install.
+
+  The `MIX_INSTALL_RESTORE_PROJECT_DIR` environment variable may be specified
+  since Elixir v1.16.2. It should point to a previous installation directory,
+  which can be obtained with `Mix.install_project_dir/0` (after calling `Mix.install/2`).
+  Using a restore dir may speed up the installation, since matching dependencies
+  do not need be refetched nor recompiled. This environment variable is ignored
+  if `:force` is enabled.
   """
   @doc since: "1.12.0"
   def install(deps, opts \\ [])
@@ -796,11 +827,25 @@ defmodule Mix do
           other
       end)
 
-    config = Keyword.get(opts, :config, [])
+    opts =
+      Keyword.validate!(opts,
+        config: [],
+        config_path: nil,
+        consolidate_protocols: true,
+        elixir: nil,
+        force: false,
+        lockfile: nil,
+        runtime_config: [],
+        start_applications: true,
+        system_env: [],
+        verbose: false
+      )
+
     config_path = expand_path(opts[:config_path], deps, :config_path, "config/config.exs")
-    system_env = Keyword.get(opts, :system_env, [])
-    consolidate_protocols? = Keyword.get(opts, :consolidate_protocols, true)
-    start_applications? = Keyword.get(opts, :start_applications, true)
+    config = Keyword.fetch!(opts, :config)
+    system_env = Keyword.fetch!(opts, :system_env)
+    consolidate_protocols? = Keyword.fetch!(opts, :consolidate_protocols)
+    start_applications? = Keyword.fetch!(opts, :start_applications)
 
     id =
       {deps, config, system_env, consolidate_protocols?}
@@ -808,57 +853,56 @@ defmodule Mix do
       |> :erlang.md5()
       |> Base.encode16(case: :lower)
 
-    force? = System.get_env("MIX_INSTALL_FORCE") in ["1", "true"] or !!opts[:force]
+    force? = System.get_env("MIX_INSTALL_FORCE") in ["1", "true"] or Keyword.fetch!(opts, :force)
 
     case Mix.State.get(:installed) do
       nil ->
         Application.put_all_env(config, persistent: true)
         System.put_env(system_env)
 
-        install_dir = install_dir(id)
+        install_project_dir = install_project_dir(id)
 
-        if opts[:verbose] do
-          Mix.shell().info("Mix.install/2 using #{install_dir}")
+        if Keyword.fetch!(opts, :verbose) do
+          Mix.shell().info("Mix.install/2 using #{install_project_dir}")
         end
 
         if force? do
-          File.rm_rf!(install_dir)
+          File.rm_rf!(install_project_dir)
         end
 
-        config = [
-          version: "0.1.0",
-          build_embedded: false,
-          build_per_environment: true,
-          build_path: "_build",
-          lockfile: "mix.lock",
-          deps_path: "deps",
+        dynamic_config = [
           deps: deps,
-          app: :mix_install,
-          erlc_paths: [],
-          elixirc_paths: [],
-          compilers: [],
           consolidate_protocols: consolidate_protocols?,
-          config_path: config_path,
-          prune_code_paths: false
+          config_path: config_path
         ]
 
+        config = install_project_config(dynamic_config)
+
         started_apps = Application.started_applications()
         :ok = Mix.ProjectStack.push(@mix_install_project, config, "nofile")
-        build_dir = Path.join(install_dir, "_build")
+        build_dir = Path.join(install_project_dir, "_build")
         external_lockfile = expand_path(opts[:lockfile], deps, :lockfile, "mix.lock")
 
         try do
           first_build? = not File.dir?(build_dir)
-          File.mkdir_p!(install_dir)
 
-          File.cd!(install_dir, fn ->
+          restore_dir = System.get_env("MIX_INSTALL_RESTORE_PROJECT_DIR")
+
+          if first_build? and restore_dir != nil and not force? do
+            File.cp_r(restore_dir, install_project_dir)
+            remove_dep(install_project_dir, @mix_install_app_string)
+          end
+
+          File.mkdir_p!(install_project_dir)
+
+          File.cd!(install_project_dir, fn ->
             if config_path do
               Mix.Task.rerun("loadconfig")
             end
 
             cond do
               external_lockfile ->
-                md5_path = Path.join(install_dir, "merge.lock.md5")
+                md5_path = Path.join(install_project_dir, "merge.lock.md5")
 
                 old_md5 =
                   case File.read(md5_path) do
@@ -869,7 +913,7 @@ defmodule Mix do
                 new_md5 = external_lockfile |> File.read!() |> :erlang.md5()
 
                 if old_md5 != new_md5 do
-                  lockfile = Path.join(install_dir, "mix.lock")
+                  lockfile = Path.join(install_project_dir, "mix.lock")
                   old_lock = Mix.Dep.Lock.read(lockfile)
                   new_lock = Mix.Dep.Lock.read(external_lockfile)
                   Mix.Dep.Lock.write(Map.merge(old_lock, new_lock), file: lockfile)
@@ -907,13 +951,22 @@ defmodule Mix do
             end
           end
 
-          Mix.State.put(:installed, id)
+          if restore_dir do
+            remove_leftover_deps(install_project_dir)
+          end
+
+          Mix.State.put(:installed, {id, dynamic_config})
           :ok
         after
           Mix.ProjectStack.pop()
+          # Clear all tasks invoked during installation, since there
+          # is no reason to keep this in memory. Additionally this
+          # allows us to rerun tasks for the dependencies later on,
+          # such as recompilation
+          Mix.Task.clear()
         end
 
-      ^id when not force? ->
+      {^id, _dynamic_config} when not force? ->
         :ok
 
       _ ->
@@ -939,7 +992,35 @@ defmodule Mix do
     Path.join(app_dir, relative_path)
   end
 
-  defp install_dir(cache_id) do
+  defp remove_leftover_deps(install_project_dir) do
+    build_lib_dir = Path.join([install_project_dir, "_build", "dev", "lib"])
+
+    deps = File.ls!(build_lib_dir)
+
+    loaded_deps =
+      for {app, _description, _version} <- Application.loaded_applications(),
+          into: MapSet.new(),
+          do: Atom.to_string(app)
+
+    # We want to keep :mix_install, but it has no application
+    loaded_deps = MapSet.put(loaded_deps, @mix_install_app_string)
+
+    for dep <- deps, not MapSet.member?(loaded_deps, dep) do
+      remove_dep(install_project_dir, dep)
+    end
+  end
+
+  defp remove_dep(install_project_dir, dep) do
+    build_lib_dir = Path.join([install_project_dir, "_build", "dev", "lib"])
+    deps_dir = Path.join(install_project_dir, "deps")
+
+    build_path = Path.join(build_lib_dir, dep)
+    File.rm_rf(build_path)
+    dep_path = Path.join(deps_dir, dep)
+    File.rm_rf(dep_path)
+  end
+
+  defp install_project_dir(cache_id) do
     install_root =
       System.get_env("MIX_INSTALL_DIR") ||
         Path.join(Mix.Utils.mix_cache(), "installs")
@@ -948,12 +1029,74 @@ defmodule Mix do
     Path.join([install_root, version, cache_id])
   end
 
+  defp install_project_config(dynamic_config) do
+    [
+      version: "0.1.0",
+      build_embedded: false,
+      build_per_environment: true,
+      build_path: "_build",
+      lockfile: "mix.lock",
+      deps_path: "deps",
+      app: @mix_install_app,
+      erlc_paths: [],
+      elixirc_paths: [],
+      compilers: [],
+      prune_code_paths: false
+    ] ++ dynamic_config
+  end
+
+  @doc false
+  def in_install_project(fun) do
+    case safe_get_installed() do
+      {id, dynamic_config} ->
+        config = install_project_config(dynamic_config)
+
+        install_project_dir = install_project_dir(id)
+
+        File.cd!(install_project_dir, fn ->
+          :ok = Mix.ProjectStack.push(@mix_install_project, config, "nofile")
+
+          try do
+            fun.()
+          after
+            Mix.ProjectStack.pop()
+          end
+        end)
+
+      nil ->
+        Mix.raise("trying to call Mix.in_install_project/1, but Mix.install/2 was never called")
+    end
+  end
+
+  @doc """
+  Returns the directory where the current `Mix.install/2` project
+  resides.
+  """
+  @doc since: "1.16.2"
+  @spec install_project_dir() :: Path.t() | nil
+  def install_project_dir() do
+    case safe_get_installed() do
+      {id, _dynamic_config} -> install_project_dir(id)
+      nil -> nil
+    end
+  end
+
   @doc """
   Returns whether `Mix.install/2` was called in the current node.
   """
   @doc since: "1.13.0"
   def installed? do
-    Mix.State.get(:installed) != nil
+    safe_get_installed() != nil
+  end
+
+  defp safe_get_installed() do
+    if mix_started?() do
+      Mix.State.get(:installed)
+    end
+  end
+
+  defp mix_started?() do
+    Process.whereis(Mix.State) != nil
   end
 
   defp stop_apps([]), do: :ok
