@@ -178,13 +178,6 @@ defmodule Mix.Dep do
     deps
   end
 
-  @doc """
-  Although private API, this is kept to avoid breaking code on patch releases.
-  """
-  def load_on_environment(opts) do
-    Mix.Dep.Converger.converge(opts)
-  end
-
   @doc """
   Clears loaded dependencies from the cache for the current environment.
   """
