@@ -68,7 +68,7 @@ defmodule Mix.Release do
   @safe_modes [:permanent, :temporary, :transient]
   @unsafe_modes [:load, :none]
   @additional_chunks ~w(Attr)c
-  @copy_app_dirs ["priv"]
+  @copy_app_dirs ["priv", "include"]
 
   @doc false
   @spec from_config!(atom, keyword, keyword) :: t
@@ -392,7 +392,7 @@ defmodule Mix.Release do
   defp boot_mode(mode), do: mode
 
   defp build_start_clean_boot(boot) do
-    for({app, _mode} <- boot, do: {app, :none})
+    for({app, mode} <- boot, do: {app, if(mode == :none, do: :none, else: :load)})
     |> Keyword.put(:stdlib, :permanent)
     |> Keyword.put(:kernel, :permanent)
   end
@@ -755,6 +755,7 @@ defmodule Mix.Release do
   @doc """
   Finds a template path for the release.
   """
+  @spec rel_templates_path(t, Path.t()) :: binary
   def rel_templates_path(release, path) do
     Path.join(release.options[:rel_templates_path] || "rel", path)
   end
@@ -762,7 +763,7 @@ defmodule Mix.Release do
   @doc """
   Copies ERTS if the release is configured to do so.
 
-  Returns true if the release was copied, false otherwise.
+  Returns `true` if the release was copied, `false` otherwise.
   """
   @spec copy_erts(t) :: boolean()
   def copy_erts(%{erts_source: nil}) do
