@@ -46,15 +46,26 @@ defmodule Mix.Task do
 
   ## Module attributes
 
+  You can control some behavior of your Mix task by setting module
+  attributes. This section documents the available attributes.
+
+  ### `@shortdoc`
+
   Define the `@shortdoc` attribute if you wish to make the task
   publicly visible on `mix help`. Omit this attribute if you do
   not want your task to be listed via `mix help`.
+
+  ### `@moduledoc`
+
   The `@moduledoc` attribute may override `@shortdoc`. The task
   will not appear in `mix help` if documentation for the entire
   module is hidden with `@moduledoc false`.
 
-  If a task has requirements, they can be listed using the
-  `@requirements` attribute. For example:
+  ### `@requirements`
+
+  If a task has *requirements*, they can be listed using the
+  `@requirements` attribute. Requirements are other Mix
+  tasks that this task requires to have run. For example:
 
       @requirements ["app.config"]
 
@@ -75,9 +86,18 @@ defmodule Mix.Task do
     * "app.start" - additionally starts the supervision tree of
       the current project and its dependencies
 
-  Finally, set `@recursive true` if you want the task to run
+  ### `@recursive`
+
+  Set `@recursive true` if you want the task to run
   on each umbrella child in an umbrella project.
 
+  ### `@preferred_cli_env`
+
+  Sets the preferred Mix environment for this task. For example,
+  if your task is meant to be used for testing, you could set
+
+      @preferred_cli_env :test
+
   ## Documentation
 
   Users can read the documentation for public Mix tasks by
@@ -210,7 +230,7 @@ defmodule Mix.Task do
   @doc """
   Indicates if the current task is recursing.
 
-  This returns true if a task is marked as recursive
+  This returns `true` if a task is marked as recursive
   and it is being executed inside an umbrella project.
   """
   @doc since: "1.8.0"
@@ -219,6 +239,9 @@ defmodule Mix.Task do
     Mix.ProjectStack.recursing() != nil
   end
 
+  @doc """
+  Available for backwards compatibility.
+  """
   @deprecated "Configure the environment in your mix.exs"
   defdelegate preferred_cli_env(task), to: Mix.CLI
 
@@ -254,7 +277,7 @@ defmodule Mix.Task do
   @doc """
   Checks if the given `task` name is an alias.
 
-  Returns false if the given name is not an alias or if it is not a task.
+  Returns `false` if the given name is not an alias or if it is not a task.
 
   For more information about task aliasing, take a look at the
   ["Aliases"](https://hexdocs.pm/mix/Mix.html#module-aliases) section in the
@@ -472,14 +495,21 @@ defmodule Mix.Task do
   end
 
   defp with_debug(task, args, proj, fun) do
-    if Mix.debug?() do
-      shell = Mix.shell()
-      shell.info(["-> Running mix ", task_to_string(task, args), project_to_string(proj)])
-      {time, res} = :timer.tc(fun)
-      shell.info(["<- Ran mix ", task, " in ", Integer.to_string(div(time, 1000)), "ms"])
-      res
-    else
-      fun.()
+    cond do
+      Mix.debug?() ->
+        shell = Mix.shell()
+        shell.info(["-> Running mix ", task_to_string(task, args), project_to_string(proj)])
+        {time, res} = :timer.tc(fun)
+        shell.info(["<- Ran mix ", task, " in ", Integer.to_string(div(time, 1000)), "ms"])
+        res
+
+      task in Mix.State.get(:profile, []) ->
+        shell = Mix.shell()
+        shell.info(["-> Profiling mix ", task_to_string(task, args), project_to_string(proj)])
+        Mix.Tasks.Profile.Eprof.profile(fun, warmup: false, set_on_spawn: false)
+
+      true ->
+        fun.()
     end
   end
 
