@@ -106,6 +106,9 @@ defmodule Mix.Tasks.Test do
 
   ## Command line options
 
+    * `--all-warnings` (`--no-all-warnings`) - prints all warnings, including previous compilations
+      (default is true except on errors)
+
     * `--color` - enables color in the output
 
     * `--cover` - runs coverage tool. See "Coverage" section below
@@ -137,8 +140,6 @@ defmodule Mix.Tasks.Test do
     * `--max-failures` - the suite stops evaluating tests when this number of test
       failures is reached. It runs all tests if omitted
 
-    * `--no-all-warnings` - prints only warnings from files currently compiled (instead of all)
-
     * `--no-archives-check` - does not check archives
 
     * `--no-color` - disables color in the output
@@ -292,7 +293,7 @@ defmodule Mix.Tasks.Test do
       invoked during the tests themselves. That's because macros are invoked at
       compilation time, before the test coverage instrumentation begins;
 
-  ### Configuratiuon
+  ### Configuration
 
   The `:test_coverage` configures the coverage tool and accepts the following options:
 
@@ -387,7 +388,7 @@ defmodule Mix.Tasks.Test do
 
   ## File-system watchers
 
-  You can integrate `mix test` with filesystem watchers through the command line
+  You can integrate `mix test` with file system watchers through the command line
   via the `--listen-on-stdin` option. For example, you can use [fswatch](https://github.com/emcrisostomo/fswatch)
   or similar to emit newlines whenever there is a change, which will cause your test
   suite to re-run:
@@ -475,7 +476,7 @@ defmodule Mix.Tasks.Test do
   end
 
   defp relative_app_file_exists?(file) do
-    {file, _} = ExUnit.Filters.parse_path(file)
+    {[file], _} = ExUnit.Filters.parse_paths([file])
     File.exists?(Path.join("../..", file))
   end
 
@@ -554,7 +555,7 @@ defmodule Mix.Tasks.Test do
 
     # Finally parse, require and load the files
     test_elixirc_options = project[:test_elixirc_options] || []
-    test_files = parse_files(files, shell, test_paths)
+    test_files = if files != [], do: parse_file_paths(files), else: test_paths
     test_pattern = project[:test_pattern] || "*_test.exs"
     warn_test_pattern = project[:warn_test_pattern] || "*_test.ex"
 
@@ -691,24 +692,10 @@ defmodule Mix.Tasks.Test do
     [autorun: false] ++ opts
   end
 
-  defp parse_files([], _shell, test_paths) do
-    test_paths
-  end
-
-  defp parse_files([single_file], _shell, _test_paths) do
-    # Check if the single file path matches test/path/to_test.exs:123. If it does,
-    # apply "--only line:123" and trim the trailing :123 part.
-    {single_file, opts} = ExUnit.Filters.parse_path(single_file)
-    ExUnit.configure(opts)
-    [single_file]
-  end
-
-  defp parse_files(files, shell, _test_paths) do
-    if Enum.any?(files, &match?({_, [_ | _]}, ExUnit.Filters.parse_path(&1))) do
-      raise_with_shell(shell, "Line numbers can only be used when running a single test file")
-    else
-      files
-    end
+  defp parse_file_paths(file_paths) do
+    {parsed_file_paths, ex_unit_opts} = ExUnit.Filters.parse_paths(file_paths)
+    ExUnit.configure(ex_unit_opts)
+    parsed_file_paths
   end
 
   defp parse_filters(opts, key) do
